"""
WebPortfolio command line tool

manage.py

Command line tool to manage your application

"""

import click
from webportfolio import get_env_config
import webportfolio.utils

from application import config, model
from webportfolio.module.user import PRIMARY_ROLES as USER_PRIMARY_ROLES

conf = get_env_config(config)

@click.group()
def cli():
    pass

@cli.command()
def setup():
    """
    Setup
    :return:
    """

    # Create all db
    model.db.create_all()

    # :: USERS
    # Setup primary roles.
    # PRIMARY ROLES is a set of tuples [(level, name), ...]
    [model.User.Role.new(level=r[0], name=r[1]) for r in USER_PRIMARY_ROLES]

    # ADD SUPER ADMIN
    if not hasattr(conf, "APPLICATION_ADMIN_EMAIL") \
        or not conf.APPLICATION_ADMIN_EMAIL \
        or conf.APPLICATION_ADMIN_EMAIL == "" \
        or not webportfolio.utils.is_valid_email(conf.APPLICATION_ADMIN_EMAIL):
        raise Exception("APPLICATION_ADMIN_EMAIL is empty or not valid")
    else:
        random_password = webportfolio.utils.generate_random_string()
        admin_email = conf.APPLICATION_ADMIN_EMAIL
        user = model.User.User.get_by_email(admin_email)
        if not user:
            model.User.User.new(email=admin_email,
                                password=random_password,
                                first_name="Admin",
                                role="superadmin")
            click.echo("")
            click.echo("** New Super Admin Created")
            click.echo("- Admin Email: %s" % admin_email)
            click.echo("- Admin Password: %s" % random_password)
            click.echo("")

    # :: POSTS
    # Set types
    post_types = ["Page", "Blog", "Document", "Other"]
    if not model.Cms.Type.all().count():
        click.echo(" Creating CMS Post Types...")
        [model.Cms.Type.new(t) for t in post_types]

    # Set categories
    post_categories = ["Uncatgorized"]
    if not model.Cms.Category.all().count():
        click.echo(" Creating CMS Post Categories...")
        [model.Cms.Category.new(c) for c in post_categories]

    # Add some basic post
    if not model.Cms.Post.all().count():
        click.echo(" Creating standard CMS Posts...")
        user = model.User.User.get_by_email(admin_email)
        if not user:
            raise Exception("Can't insert new Posts. User doesn't exist")
        posts = [
            {
                "title": "About Us",
                "slug": "about",
                "content": "**About Us**",
                "type_slug": "document",
                "is_published": True,
                "user_id": user.id,
            },
            {
                "title": "Terms of Service",
                "slug": "tos",
                "content": "**Terms of Service**",
                "type_slug": "document",
                "is_published": True,
                "user_id": user.id,
            },
            {
                "title": "Privacy",
                "slug": "privacy",
                "content": "**Privacy Policy**",
                "type_slug": "document",
                "is_published": True,
                "user_id": user.id,
            },
            {
                "title": "First Page",
                "slug": "first-page",
                "content": "**This is our First Page!**",
                "type_slug": "page",
                "is_published": True,
                "user_id": user.id,
            },
            {
                "title": "First Blog",
                "slug": "first-blog",
                "content": "**This is our First Blog!**",
                "type_slug": "blog",
                "is_published": True,
                "user_id": user.id,
            }
        ]
        [model.Cms.Post.new(**post) for post in posts]

if __name__ == "__main__":
    click.echo("WebPortfolio Manager")

    cli()






"""
Contact Page
"""

from flask import request, redirect, url_for, abort
from webportfolio import (WebPortfolio, route, mailer, recaptcha, nav_menu,
                       flash_error, flash_success, warn_missing_config)
import webportfolio.utils as utils

WebPortfolio.register_module(__name__)

def contact_page(view, **kwargs):
    """
    :param view: The view to copy to
    :param kwargs:
        nav_menu: dict(contact="Contact")
        route: Not available

    :return:
    """
    opt_route = kwargs.pop("route", {})
    opt_nav_menu = kwargs.pop("nav_menu", {})

    template_dir = kwargs.pop("template_dir", "WebPortfolio/Module/ContactPage")
    template_page = template_dir + "/%s.html"

    @nav_menu(opt_nav_menu.get("contact", "Contact"),
              order=100,
              endpoint="ContactPage",
              __is_class=False,
              __module=view.__module__,
              __class=view.__name__)
    @route("contact", methods=["GET", "POST"], endpoint="ContactPage")
    def contact_page(self):
        if not self.config("MAILER_URI") \
                or not self.config("MODULE_CONTACT_PAGE_EMAIL"):
            abort(500, "Mailer Error. Invalid [ MAILER_URI ] "
                       "or [ MODULE_CONTACT_PAGE_EMAIL ] is missing or empty")

        contact_email = self.config("MODULE_CONTACT_PAGE_EMAIL")

        if request.method == "POST":
            error_message = None
            email = request.form.get("email")
            subject = request.form.get("subject")
            message = request.form.get("message")
            name = request.form.get("name")

            if recaptcha.verify():
                if not email or not subject or not message:
                    error_message = "All fields are required"
                elif not utils.is_valid_email(email):
                    error_message = "Invalid email address"
                if error_message:
                    flash_error(error_message)
                else:
                    mailer.send_template("contact-us.txt",
                                         to=contact_email,
                                         reply_to=email,
                                         mail_from=email,
                                         mail_subject=subject,
                                         mail_message=message,
                                         mail_name=name
                                        )
                    flash_success("Message sent. Thank you!")
            else:
                flash_error("Security code is invalid")
            return redirect(url_for("ContactPage"))
        else:
            self.page_meta(title="Contact Us")
            return self.render(view_template=template_page % "contact_page")

    return contact_page





"""
::WebPortfolio::

serve_www.py

This is the entry point of the application.


"""

from webportfolio import WebPortfolio, get_env

# Import the application's views
import application.www.views

# The directory containing your views/webportfolio/templates
app_dir = "application/www"

# The project config object
app_config = "application.config.%s" % get_env()

# WebPortfolio
# init() returns a flask object instance
WP = WebPortfolio()
app = WP.init(__name__, directory=app_dir, config=app_config)

if __name__ == "__main__":
    app.run(debug=True)



"""
CMS

"""
import os
import datetime
import json
from flask import url_for, request, redirect, abort, session, jsonify
from webportfolio import WebPortfolio, route, storage, nav_menu, \
    flash_success, flash_data, get_flashed_data, flash_error, ModelError
from webportfolio import utils, wp_markdown
from flask_login import login_required, current_user
from paginator import Paginator

WebPortfolio.register_module(__name__)

def model(UserModel):
    """
    Post Model
    :param UserModel:
    """

    db = UserModel.User.db

    class SlugNameMixin(object):
        name = db.Column(db.String(250), index=True)
        slug = db.Column(db.String(250), index=True, unique=True)
        description = db.Column(db.String(250))
        image_url = db.Column(db.Text)

        @classmethod
        def get_by_slug(cls, slug=None, name=None):
            """
            Return a post by slug
            """
            if name and not slug:
                slug = utils.slugify(name)
            return cls.all().filter(cls.slug == slug).first()

        @classmethod
        def new(cls, name, slug=None):
            slug = utils.slugify(name if not slug else slug)
            return cls.create(name=name, slug=slug)

        def rename(self, name, slug=None):
            slug = utils.slugify(name if not slug else slug)
            return self.update(name=name, slug=slug)

    class CmsPostType(SlugNameMixin, db.Model):
        """
        Types
        """
        @property
        def total_posts(self):
            return CmsPost.all().filter(CmsPost.type_id == self.id).count()

    class CmsPostCategory(SlugNameMixin, db.Model):
        """
        Category
        """
        @property
        def total_posts(self):
            return CmsPostCategoryMap.all()\
                .filter(CmsPostCategoryMap.category_id == self.id)\
                .count()

    class CmsPostTag(SlugNameMixin, db.Model):
        """
        Tag
        """
        @property
        def total_posts(self):
            return CmsPostTagMap.all()\
                .filter(CmsPostTagMap.tag_id == self.id)\
                .count()

    class CmsPostTagMap(db.Model):
        """
        PostPostTag
        """
        post_id = db.Column(db.Integer, db.ForeignKey("cms_post.id"))
        tag_id = db.Column(db.Integer, db.ForeignKey(CmsPostTag.id))

        @classmethod
        def add(cls, post_id, tag_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.tag_id == tag_id)\
                .first()
            if not c:
                cls.create(post_id=post_id, tag_id=tag_id)

        @classmethod
        def remove(cls, post_id, tag_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.tag_id == tag_id)\
                .first()
            if c:
                c.delete(hard_delete=True)

    class CmsPostCategoryMap(db.Model):
        post_id = db.Column(db.Integer, db.ForeignKey("cms_post.id"))
        category_id = db.Column(db.Integer, db.ForeignKey(CmsPostCategory.id))

        @classmethod
        def add(cls, post_id, category_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.category_id == category_id)\
                .first()
            if not c:
                cls.create(post_id=post_id, category_id=category_id)

        @classmethod
        def remove(cls, post_id, category_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.category_id == category_id)\
                .first()
            if c:
                c.delete(hard_delete=True)

    class CmsPost(db.Model):

        user_id = db.Column(db.Integer, db.ForeignKey(UserModel.User.id))
        type_id = db.Column(db.Integer, db.ForeignKey(CmsPostType.id))

        title = db.Column(db.String(250))
        slug = db.Column(db.String(250), index=True)
        content = db.Column(db.Text)
        description = db.Column(db.Text)
        featured_image = db.Column(db.Text)
        featured_embed = db.Column(db.Text)
        language = db.Column(db.String(250))
        parent_id = db.Column(db.Integer)  # If the post is derived from another post
        is_child = db.Column(db.Boolean, index=True, default=False)  #
        is_list = db.Column(db.Boolean, index=True, default=False)  # A list is a type of post having sub post
        is_featured = db.Column(db.Boolean, index=True, default=False)  # Feature post are limited
        featured_at = db.Column(db.DateTime)
        is_sticky = db.Column(db.Boolean, index=True, default=False)  # A sticky post usually stay on top, no matter the count
        sticky_at = db.Column(db.DateTime)
        is_published = db.Column(db.Boolean, index=True, default=True)
        published_at = db.Column(db.DateTime)
        published_by = db.Column(db.Integer)
        is_revision = db.Column(db.Boolean, default=False)
        revision_id = db.Column(db.Integer)  # When updating the post, will auto-save

        is_public = db.Column(db.Boolean, index=True, default=False)
        is_draft = db.Column(db.Boolean, index=True, default=False)
        options_data = db.Column(db.Text, default="{}")
        menu_order = db.Column(db.Integer, default=0, index=True)

        author = db.relationship(UserModel.User, backref="posts")
        type = db.relationship(CmsPostType, backref="posts")
        categories = db.relationship(CmsPostCategory,
                                     secondary=CmsPostCategoryMap.__table__.name)
        tags = db.relationship(CmsPostTag,
                                     secondary=CmsPostTagMap.__table__.name)

        @classmethod
        def new(cls, title, **kwargs):
            """
            Insert a new post
            """
            published_date = None
            is_revision = False
            is_published = False
            is_draft = False
            is_public = kwargs.get("is_public", True)
            parent_id = kwargs.get("parent_id", None)

            if kwargs.get("is_revision"):
                if not parent_id:
                    raise ModelError("'parent_id' is missing for revision")
                is_revision = True
                is_public = False
            elif kwargs.get("is_draft"):
                is_draft = True
                is_public = False
            elif kwargs.get("is_published"):
                is_published = True
                published_date = datetime.datetime.now()

            slug = None
            if is_published or is_draft:
                slug = cls.create_slug(kwargs.get("slug", title))

            type_id = kwargs.get("type_id")
            if not type_id and kwargs.get("type_slug"):
                type_slug = kwargs.get("type_slug")
                _type = CmsPostType.get_by_slug(slug=type_slug)
                if _type:
                    type_id = _type.id

            data = {
                "user_id": kwargs.get("user_id", 0),
                "title": title,
                "slug": slug,
                "content": kwargs.get("content"),
                "description": kwargs.get("description"),
                "is_published": is_published,
                "published_at": published_date,
                "is_draft": is_draft,
                "is_revision": is_revision,
                "is_public": is_public,
                "parent_id": parent_id,
                "type_id": type_id
            }
            return cls.create(**data)

        @classmethod
        def get_published(cls, id=None, slug=None, types=[], categories=[], tags=[]):
            """
            Return published posts.
            If $id or $slug it will return a single post, else all

            :param id: int - the id of a post
            :param slug: string - the slug of a post
            :param types: list - list of types slugs
            :param categories: list - list of categories slug
            :param tags: list - list of tags slugs
            :return:
            """
            q = cls.all().filter(cls.is_published == True)

            # Query only a single post
            if id or slug:
                if id:
                    q = q.filter(cls.id == id)
                elif slug:
                    q = q.filter(cls.slug == slug)
                return q.first()

            # Query lists
            else:
                if types:
                    q = q.join(CmsPostType)\
                        .filter(CmsPostType.slug.in_(types))
                if categories:
                    q = q.join(CmsPostCategoryMap)\
                        .join(CmsPostCategory)\
                        .filter(CmsPostCategory.slug.in_(categories))
                if tags:
                    q = q.join(CmsPostTag)\
                        .filter(CmsPostTag.slug.in_(tags))
                return q

        @classmethod
        def create_slug(cls, title):
            slug = None
            slug_counter = 0
            _slug = utils.slugify(title).lower()
            while True:
                slug = _slug
                if slug_counter > 0:
                    slug += str(slug_counter)
                    slug_counter += 1
                if not cls.get_by_slug(slug):
                    break
            return slug

        @classmethod
        def get_by_slug(cls, slug):
            """
            Return a post by slug
            """
            return cls.all().filter(cls.slug == slug).first()

        def publish(self, published_date=None, published_by_id=None):
            if self.is_draft:
                data = {
                    "is_draft": False,
                    "is_published": True,
                    "published_at": published_date or datetime.datetime.now()
                }
                if published_by_id:
                    data.update({
                        "published_by": published_by_id
                    })

                self.update(**data)

        def set_slug(self, title):
            slug = utils.slugify(title)
            if title and slug != self.slug:
                slug = self.create_slug(slug)
                self.update(slug=slug)

        def update_categories(self, categories_list):
            """
            Update categories by replacing existing list with new list
            :param categories_list: list. The new list of category
            """
            cats = CmsPostCategoryMap.all()\
                    .filter(CmsPostCategoryMap.post_id == self.id)
            cats_list = [c.category_id for c in cats]

            del_cats = list(set(cats_list) - set(categories_list))
            new_cats = list(set(categories_list) - set(cats_list))

            for dc in del_cats:
                CmsPostCategoryMap.remove(post_id=self.id, category_id=dc)

            for nc in new_cats:
                CmsPostCategoryMap.add(post_id=self.id, category_id=nc)

        def update_tags(self, tags_list):
            """
            Update tags by replacing existing list with new list
            :param tags_list: list. The new list of tags
            """
            tags = CmsPostTagMap.all()\
                    .filter(CmsPostTagMap.post_id == self.id)
            tags_list_ = [c.tag_id for c in tags]

            del_tags = list(set(tags_list_) - set(tags_list))
            new_tags = list(set(tags_list) - set(tags_list_))

            for dc in del_tags:
                CmsPostTagMap.remove(post_id=self.id, tag_id=dc)

            for nc in new_tags:
                CmsPostTagMap.add(post_id=self.id, tag_id=nc)

        def get_list(self):
            if not self.is_list:
                return None

            return CmsPost.all()\
                .filter(CmsPost.is_published == True)\
                .filter(CmsPost.is_child == True)\
                .filter(CmsPost.parent_id == self.id)

        def delete_revisions(self):
            """
            Delete all revisions
            """
            try:
                CmsPost.all()\
                    .filter(CmsPost.post_id == self.id)\
                    .filter(CmsPost.is_revision == True)\
                    .delete()
                CmsPost.db.commit()
            except Exception as ex:
                CmsPost.db.rollback()

        def set_options(self, key, values):
            options = self.options
            options.update({key: values})
            self.update(options_data=json.dumps(options))

        @property
        def options(self):
            return json.loads(self.options_data) if self.options_data else {}

        @property
        def excerpt(self):
            """
            Return description as excerpt, if empty,
            it will return the first paragraph
            :return: str
            """
            if self.description:
                return self.description
            else:
                return ""

        @property
        def top_image(self):
            """
            Return the top image
            Return the image url if exists, or it will get the first image
            Will get the first image from the markdown
            """
            if self.featured_image:
                return self.featured_image
            elif self.content:
                md_images = wp_markdown.extract_images(self.content)
                return md_images[0] if md_images else None
            return None

        @property
        def status(self):
            if self.is_published:
                return "Published"
            elif self.is_draft:
                return "Draft"
            elif self.is_revision:
                return "Revision"
            else:
                return ""

        @property
        def total_revisions(self):
            return CmsPost.all()\
                .filter(CmsPost.post_id == self.id)\
                .filter(CmsPost.is_revision == True)\
                .count()

    class CmsUploadObject(db.Model):
        parent_id = db.Column(db.Integer, index=True)
        user_id = db.Column(db.Integer, index=True)
        provider = db.Column(db.String(250))
        container = db.Column(db.String(250))
        local_path = db.Column(db.Text)
        name = db.Column(db.Text)
        description = db.Column(db.String(250))
        size = db.Column(db.Integer)
        extension = db.Column(db.String(10), index=True)
        type = db.Column(db.String(25), index=True)
        object_path = db.Column(db.Text)
        object_url = db.Column(db.Text)
        is_private = db.Column(db.Boolean, index=True, default=False)

    return utils.to_struct(Post=CmsPost,
                           Category=CmsPostCategory,
                           Type=CmsPostType,
                           CategoryMap=CmsPostCategoryMap,
                           Tag=CmsPostTag,
                           TagMap=CmsPostTagMap,
                           UploadObject=CmsUploadObject)

# ------------------------------------------------------------------------------

def post(view, **kwargs):
    """
    It extends your view  read, list content from the CMS

    kwargs:
        model: class
        templates_dir: str - The directory of templates
        query: dict
            - types: list of types to show
            - order_by: str - of order by
        endpoints: dict
            - {
                $action_name: {
                    - endpoint: The endpoint to use for the action
                    - menu: The name of the menu
                    - show_menu: bool - show/hide menu
                    - menu_order: int - position of the menu
                    - title:
                    - image
                }
            }, ...
    """

    # Setting up endpoint config, for url based
    # It will be used to access the proper endpoint and format the
    endpoints_config = {
        "id": {
            "url": "{id}",
            "accept": ["id"]
        },
        "slug": {
            "url": "{slug}",
            "accept": ["slug"]
        },
        "id-slug": {
            "url": "{id}/{slug}",
            "accept": ["id", "slug"]
        },
        "month-slug": {
            "url": "{month}/{slug}",
            "accept": ["month", "slug"]
        },
        "date-slug": {
            "url": "{date}/{slug}",
            "accept": ["date", "slug"]
        }
    }
    endpoints_config = utils.DotDict(endpoints_config)

    # Totally required
    model = kwargs.get("model")

    # templates directory
    templates_dir = kwargs.get("templates_dir", "WebPortfolio/Module/Cms/Post")

    # query
    opt_queries = utils.DotDict(kwargs.get("query", {}))
    post_types = opt_queries.get("types", [])
    post_categories = opt_queries.get("categories", [])
    post_tags = opt_queries.get("categories", [])
    post_order_by = opt_queries.get("order_by", "published_at desc")
    post_per_page = opt_queries.get("per_page", 20)

    # endpoints
    opt_endpoints = utils.DotDict(kwargs.get("endpoints", {}))

    # The single.endpoint accept an extra param to format the post read
    # :slug
    # endpoint:url
    single_endpoint_url = "slug"
    single_endpoint = opt_endpoints.get("single.endpoint", "post:slug")
    if ":" in single_endpoint:
        single_endpoint, single_endpoint_url = single_endpoint.split(":", 2)
    single_url_option = endpoints_config.get(single_endpoint_url, "slug")

    # properties to append to nav menu to attach it to the view class instead
    nav_menu_prop = {"__class": view.__name__, "__module": view.__module__}

    view_name = view.__name__

    Post = model.Cms.Post

    class Postpage(object):

        @classmethod
        def prepare_post(cls, post):
            """
            Prepare the post data,
            """
            url_kwargs = {
                "id": post.id,
                "slug": post.slug,
                "date": post.published_at.strftime("%Y/%m/%d"),
                "month": post.published_at.strftime("%Y/%m")
            }
            # set items not in the accept fields as None to not append in in the url
            url_kwargs = {_: None if _ not in single_url_option["accept"] else __
                          for _, __ in url_kwargs.items()}

            url = url_for("%s:%s" % (view_name, single_endpoint_url),
                          _external=True,
                          **url_kwargs)
            post.url = url
            post.author = cls.prepare_author(post.author)
            return post

        @classmethod
        def prepare_author(cls, author):
            """
            Prepare the author data
            """
            name = utils.slugify(author.name or "no-name")
            url = url_for("%s:post_author" % view_name, id=author.id, name=name, _external=True)
            author.url = url
            return author

        @classmethod
        def get_prev_next_post(cls, post, position):
            """
            Return previous or next post based on the current post
            :params post: post object
            :params position:
            """
            position = position.lower()
            if position not in ["prev", "next"]:
                 raise ValueError("Invalid position key. Must be 'prev' or 'next'")

            posts = Post.get_published(types=post_types)

            if position == "prev":
                posts = posts.filter(Post.id < post.id)
            elif position == "next":
                posts = posts.filter(Post.id > post.id)
            post = posts.first()
            return cls.prepare_post(post) if post else None

        @nav_menu(opt_endpoints.get("index.menu", "All Post"),
                  show=opt_endpoints.get("index.show_menu", True),
                  order=opt_endpoints.get("index.menu_order", 90),
                  **nav_menu_prop)
        @route(opt_endpoints.get("index.endpoint", "posts"))
        def post_index(self):
            """
            Endpoints options:
                index:
                    - menu
                    - show_menu
                    - menu_order
                    - endpoint
                    - per_page
                    - title
                    - post_header
                    - post_subheader
                    - post_show_byline
            """
            page = request.args.get("page", 1)
            app_per_page = self.config("APPLICATION_PAGINATION_PER_PAGE", 10)
            per_page = opt_endpoints.get("index.per_page", app_per_page)

            self.page_meta(title=opt_endpoints.get("index.title", "All Posts"))

            _query = {"types": post_types,
                      "categories": post_categories,
                      "tags": post_tags}

            posts = Post.get_published(**_query).order_by(post_order_by)
            posts = posts.paginate(page=page,
                                   per_page=per_page,
                                   callback=self.prepare_post)
            _kwargs = {
                "post_header": opt_endpoints.get("index.post_header", None),
                "post_subheader": opt_endpoints.get("index.post_subheader", None),
                "post_show_byline": opt_endpoints.get("index.post_show_byline", True)
            }

            return self.render(posts=posts,
                               view_template="%s/post_index.html" % templates_dir,
                               **_kwargs)

        @nav_menu("Post", show=False, **nav_menu_prop)  # No need to show the read in the menu
        @route("%s/<int:id>" % single_endpoint, endpoint="%s:%s" % (view_name, "id"))  # %id
        @route("%s/<slug>" % single_endpoint, endpoint="%s:%s" % (view_name, "slug"))    # %slug
        @route("%s/<int:id>/<slug>" % single_endpoint, endpoint="%s:%s" % (view_name, "id-slug")) # %id-slug
        @route('%s/<regex("[0-9]{4}/[0-9]{2}"):month>/<slug>' % single_endpoint, endpoint="%s:%s" % (view_name, "month-slug")) # %month/slug
        @route('%s/<regex("[0-9]{4}/[0-9]{2}/[0-9]{2}"):date>/<slug>' % single_endpoint, endpoint="%s:%s" % (view_name, "date-slug")) # %date/slug
        def post_single(self, id=None, slug=None, month=None, date=None):
            """
            Endpoints options
                single
                    - post_show_byline
                    - post_show_featured_image
                    - post_show_featured_embed
            """
            post = None
            if id:
                post = Post.get_published(id=id, types=post_types)
            elif slug:
                post = Post.get_published(slug=slug, types=post_types)
            if not post:
                abort(404, "Post doesn't exist")

            self.page_meta(title=post.title,
                           image=post.top_image,
                           description=post.excerpt)
            _kwargs = {
                "post_show_byline": opt_endpoints.get("single.post_show_byline", True),
                "post_show_featured_image": opt_endpoints.get("single.post_show_featured_image", True),
                "post_show_featured_embed": opt_endpoints.get("single.post_show_featured_embed", True)

            }
            return self.render(post=self.prepare_post(post),
                               prev_post=self.get_prev_next_post(post, "prev"),
                               next_post=self.get_prev_next_post(post, "next"),
                               view_template="%s/post_single.html" % templates_dir,
                               **_kwargs)

        @nav_menu(opt_endpoints.get("authors.menu", "Authors"),
                  show=opt_endpoints.get("authors.show_menu", False),
                  order=opt_endpoints.get("authors.menu_order", 91),
                  **nav_menu_prop)
        @route(opt_endpoints.get("authors.endpoint", "authors"))
        def post_authors(self):
            """
            Endpoints options
                - authors
                    menu
                    show_menu
                    menu_order
                    endpoint
                    title
            """

            self.page_meta(title=opt_endpoints.get("authors.title", "Authors"))

            authors = []
            return self.render(authors=authors,
                               view_template="%s/post_authors.html" % templates_dir)

        @nav_menu(opt_endpoints.get("author.menu", "Author"),
                  show=False,
                  **nav_menu_prop)
        @route("%s/<id>/<name>" % opt_endpoints.get("author.endpoint", "author"))
        def post_author(self, id, name=None):
            """
            Endpoints options
                - author
                    endpoint
            """

            self.page_meta(title=opt_endpoints.get("author.title", "Author"))

            author = []
            return self.render(author=author,
                               view_template="%s/post_author.html" % templates_dir)

        @nav_menu(opt_endpoints.get("archive.menu", "Archive"),
                  show=opt_endpoints.get("archive.show_menu", False),
                  order=92,
                  **nav_menu_prop)
        @route(opt_endpoints.get("archive.endpoint", "archive"))
        def post_archive(self):
            """
            Endpoints options
                - archive
                    menu
                    show_menu
                    menu_order
                    endpoint
                    title
            """
            self.page_meta(title=opt_endpoints.get("archive.title", "Archive"))

            _query = {"types": post_types,
                      "categories": post_categories,
                      "tags": post_tags}

            posts = Post.get_published(**_query)\
                .order_by(Post.published_at.desc())\
                .group_by(model.db.func.year(Post.published_at),
                          model.db.func.month(Post.published_at))

            return self.render(posts=posts,
                               view_template="%s/post_archive.html" % templates_dir)

    return Postpage

# ------------------------------------------------------------------------------

def admin(view, **kwargs):

    route_base = "cms-admin"
    menu_name = "CMS Admin"

    model = kwargs.get("model")
    templates_dir = kwargs.get("templates_dir", "WebPortfolio/Module/Cms/Admin")
    template_page = templates_dir + "/%s.html"
    PostModel = model.Cms

    # Create a CMS Admin menu for all the methods in Admin
    @nav_menu(menu_name, group="admin")
    class NavMenu(object): pass
    # The nav_menu_context helps attach all the methods to NavMenu
    nav_menu_context = dict(__module=NavMenu.__module__,
                            __class=NavMenu.__name__)
    
    class Admin(object):

        decorators = view.decorators + [login_required]

        @nav_menu("Posts", endpoint="CmsAdmin:index", order=1, **nav_menu_context)
        @route("%s" % route_base, endpoint="CmsAdmin:index")
        def cms_admin_index(self):
            """
            List all posts
            """
            self.page_meta(title="All Posts")
            per_page = self.config("APPLICATION_PAGINATION_PER_PAGE", 25)
            page = request.args.get("page", 1)
            id = request.args.get("id", None)
            slug = request.args.get("slug", None)
            status = request.args.get("status", "all")
            user_id = request.args.get("user_id", None)
            type_id = request.args.get("type_id", None)
            category_id = request.args.get("category_id", None)
            tag_id = request.args.get("tag_id", None)

            reorder_post = request.args.get("reorder_post")

            posts = PostModel.Post.all()
            types = PostModel.Type.all().order_by("name asc")

            selected_type = None
            if type_id:
                selected_type = PostModel.Type.get(type_id)

            if id:
                posts = posts.filter(PostModel.Post.id == id)
            if slug:
                posts = posts.filter(PostModel.Post.slug == slug)
            if user_id:
                posts = posts.filter(PostModel.Post.user_id == user_id)
            if type_id:
                type_id = int(type_id)
                posts = posts.filter(PostModel.Post.type_id == type_id)
            if category_id:
                posts = posts.join(PostModel.CategoryMap)\
                    .join(PostModel.Category)\
                    .filter(PostModel.Category.id == category_id)
            if tag_id:
                posts = posts.join(PostModel.TagMap)\
                    .join(PostModel.Tag)\
                    .filter(PostModel.Tag.id == tag_id)
            if status == "publish":
                posts = posts.filter(PostModel.Post.is_published == True)
            elif status == "draft":
                posts = posts.filter(PostModel.Post.is_draft == True)
            elif status == "revision":
                posts = posts.filter(PostModel.Post.is_revision == True)

            posts = posts.order_by(PostModel.Post.id.desc())
            posts = posts.paginate(page=page, per_page=per_page)

            return self.render(posts=posts,
                               types=types,
                               search=False,
                               query_vars={
                                   "id": id,
                                   "slug": slug,
                                   "user_id": user_id,
                                   "type_id": type_id,
                                   "status": status
                               },
                               selected_type=selected_type,
                               view_template=template_page % "index")

        @nav_menu("Post Preview", endpoint="CmsAdmin:preview", show=False, order=1, **nav_menu_context)
        @route("%s/preview/<id>" % route_base, endpoint="CmsAdmin:preview")
        def cms_admin_preview(self, id):
            """
            Read Post
            """
            post = PostModel.Post.get(id)
            if not post:
                abort(404, "Post doesn't exist")

            self.page_meta(title="Read: %s " % post.title)

            return self.render(post=post,
                               view_template=template_page % "preview")

        @nav_menu("Edit Post", endpoint="CmsAdmin:edit", show=False, **nav_menu_context)
        @nav_menu("New Post", endpoint="CmsAdmin:new", order=2, **nav_menu_context)
        @route("%s/new" % route_base, defaults={"id": None}, endpoint="CmsAdmin:new")
        @route("%s/edit/<id>" % route_base, endpoint="CmsAdmin:edit")
        def cms_admin_edit(self, id):
            """
            Create / Edit Post
            """
            self.page_meta(title="Edit Post")

            types = [(t.id, t.name) for t in PostModel.Type.all().order_by(PostModel.Type.name.asc())]
            categories = [(c.id, c.name) for c in PostModel.Category.all().order_by(PostModel.Category.name.asc())]
            checked_cats = []

            type_id = request.args.get("type_id", None)

            # data to pass to view
            post = {
                "id": 0,
                "title": "",
                "content": "",
                "slug": "",
                "is_public": True,
                "is_sticky": False,
                "is_featured": False,
                "type_id": 0 if not type_id else int(type_id),
                "options": {}
            }

            # saved in session
            flashed_data = get_flashed_data()
            if request.args.get("error") and flashed_data:
                post = flashed_data
                checked_cats = post["post_categories"]

            elif id:
                post = PostModel.Post.get(id)
                if not post or post.is_revision:
                    abort(404, "Post doesn't exist")
                checked_cats = [c.id for c in post.categories]

            images = PostModel.UploadObject.all()\
                .filter(PostModel.UploadObject.type == "IMAGE")\
                .order_by(PostModel.UploadObject.name.asc())

            images_list = [{"id": img.id, "url": img.object_url} for img in images]
            return self.render(post=post,
                               types=types,
                               categories=categories,
                               checked_categories=checked_cats,
                               view_template=template_page % "edit",
                               images_list=images_list)

        @route("%s/post" % route_base, methods=["POST"], endpoint="CmsAdmin:post")
        def cms_admin_post(self):
            id = request.form.get("id")
            title = request.form.get("title")
            slug = request.form.get("slug")
            content = request.form.get("content")
            description = request.form.get("description")
            type_id = request.form.get("type_id")
            post_categories = request.form.getlist("post_categories")
            published_date = request.form.get("published_date")
            status = request.form.get("status", "draft")
            is_published = True if status == "publish" else False
            is_draft = True if status == "draft" else False
            is_public = True if request.form.get("is_public") == "y" else False
            is_sticky = True if request.form.get("is_sticky") == "y" else False
            is_featured = True if request.form.get("is_featured") == "y" else False
            featured_image = request.form.get("featured_image")
            featured_embed = request.form.get("featured_embed")
            social_options = request.form.getlist("social_options")
            tags = list(set(request.form.get("tags", "").split(",")))

            if status in ["draft", "publish"] and (not title or not type_id):
                if not title:
                    flash_error("Post Title is missing ")
                if not type_id:
                    flash_error("Post type is missing")

                flash_data({
                    "id": id,
                    "title": title,
                    "content": content,
                    "description": description,
                    "slug": slug,
                    "type_id": type_id,
                    "published_date": published_date,
                    "post_categories": post_categories,
                    "featured_image": featured_image,
                    "options": {"social_options": social_options}
                })

                if id:
                    url = url_for("CmsAdmin:edit", id=id, error=1)
                else:
                    url = url_for("CmsAdmin:new", error=1)
                return redirect(url)

            data = {
                "title": title,
                "content": content,
                "description": description,
                "featured_image": featured_image,
                "featured_embed": featured_embed,
                "type_id": type_id,
                "is_sticky": is_sticky,
                "is_featured": is_featured,
                "is_public": is_public
            }

            if published_date:
                published_date = datetime.datetime.strptime(published_date, "%Y-%m-%d %H:%M:%S")
            else:
                published_date = datetime.datetime.now()

            if id and status in ["delete", "revision"]:
                post = PostModel.Post.get(id)
                if not post:
                    abort(404, "Post '%s' doesn't exist" % id)

                if status == "delete":
                    post.delete()
                    flash_success("Post deleted successfully!")
                    return redirect(url_for("CmsAdmin:index"))

                elif status == "revision":
                    data.update({
                        "user_id": current_user.id,
                        "parent_id": id,
                        "is_revision": True,
                        "is_draft": False,
                        "is_published": False,
                        "is_public": False
                    })
                    post = PostModel.Post.create(**data)
                    return jsonify({"revision_id": post.id})

            elif status in ["draft", "publish"]:
                data.update({
                    "is_published": is_published,
                    "is_draft": is_draft,
                    "is_revision": False,
                    "is_public": is_public
                })

                if id:
                    post = PostModel.Post.get(id)
                    if not post:
                        abort(404, "Post '%s' doesn't exist" % id)
                    elif post.is_revision:
                        abort(403, "Can't access this post")
                    else:
                        post.update(**data)
                else:
                    data["user_id"] = current_user.id
                    if is_published:
                        data["published_at"] = published_date
                    post = PostModel.Post.create(**data)

                # prepare tags
                _tags = []
                for tag in tags:
                    tag = tag.strip().lower()
                    _tag = PostModel.Tag.get_by_slug(name=tag)
                    if tag and not _tag:
                        _tag = PostModel.Tag.new(name=tag)
                    if _tag:
                        _tags.append(_tag.id)
                post.update_tags(_tags)

                post.set_slug(slug or title)
                post.update_categories(map(int, post_categories))
                post.set_options("social", social_options)

                if post.is_published and not post.published_at:
                    post.update(published_at=published_date)

                flash_success("Post saved successfully!")

                return redirect(url_for("CmsAdmin:edit", id=post.id))

            else:
                abort(400, "Invalid post status")

        @nav_menu("Categories", endpoint="CmsAdmin:categories", order=3, **nav_menu_context)
        @route("%s/categories" % route_base, methods=["GET", "POST"], endpoint="CmsAdmin:categories")
        def cms_admin_categories(self):
            self.page_meta(title="Post Categories")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                name = request.form.get("name")
                slug = request.form.get("slug", None)
                ajax = request.form.get("ajax", False)
                try:
                    if not id:
                        cat = PostModel.Category.new(name=name, slug=slug)
                        if ajax:
                            return jsonify({
                                "id": cat.id,
                                "name": cat.name,
                                "slug": cat.slug,
                                "status": "OK"
                            })
                        flash_success("New category '%s' added" % name)
                    else:
                        post_cat = PostModel.Category.get(id)
                        if post_cat:
                            if action == "delete":
                                post_cat.delete()
                                flash_success("Category '%s' deleted successfully!" % post_cat.name)
                            else:
                                post_cat.update(name=name, slug=slug)
                                flash_success("Category '%s' updated successfully!" % post_cat.name)
                except Exception as ex:
                    if ajax:
                        return jsonify({
                            "error": True,
                            "error_message": ex.message
                        })

                    flash_error("Error: %s" % ex.message)
                return redirect(url_for("CmsAdmin:categories"))

            else:
                cats = PostModel.Category.all().order_by(PostModel.Category.name.asc())
                return self.render(categories=cats,
                                   view_template=template_page % "categories")

        @nav_menu("Tags", endpoint="CmsAdmin:tags", order=4, **nav_menu_context)
        @route("%s/tags" % route_base, methods=["GET", "POST"], endpoint="CmsAdmin:tags")
        def cms_admin_tags(self):
            self.page_meta(title="Post Tags")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                name = request.form.get("name")
                slug = request.form.get("slug", None)
                ajax = request.form.get("ajax", False)
                try:
                    if not id:
                        tag = PostModel.Tag.new(name=name, slug=slug)
                        if ajax:
                            return jsonify({
                                "id": tag.id,
                                "name": tag.name,
                                "slug": tag.slug,
                                "status": "OK"
                            })
                        flash_success("New Tag '%s' added" % name)
                    else:
                        post_tag = PostModel.Tag.get(id)
                        if post_tag:
                            if action == "delete":
                                post_tag.delete()
                                flash_success("Tag '%s' deleted successfully!" % post_tag.name)
                            else:
                                post_tag.update(name=name, slug=slug)
                                flash_success("Tag '%s' updated successfully!" % post_tag.name)
                except Exception as ex:
                    if ajax:
                        return jsonify({
                            "error": True,
                            "error_message": ex.message
                        })

                    flash_error("Error: %s" % ex.message)
                return redirect(url_for("CmsAdmin:tags"))

            else:
                tags = PostModel.Tag.all().order_by(PostModel.Tag.name.asc())
                return self.render(tags=tags,
                                   view_template=template_page % "tags")

        @nav_menu("Types", endpoint="CmsAdmin:types", order=5, **nav_menu_context)
        @route("%s/types" % route_base, methods=["GET", "POST"], endpoint="CmsAdmin:types")
        def cms_admin_types(self):
            self.page_meta(title="Post Types")
            if request.method == "POST":
                try:
                    id = request.form.get("id", None)
                    action = request.form.get("action")
                    name = request.form.get("name")
                    slug = request.form.get("slug", None)
                    if not id:
                        PostModel.Type.new(name=name, slug=slug)
                        flash_success("New type '%s' added" % name)
                    else:
                        post_type = PostModel.Type.get(id)
                        if post_type:
                            if action == "delete":
                                post_type.delete()
                                flash_success("Type '%s' deleted successfully!" % post_type.name)
                            else:
                                post_type.update(name=name, slug=slug)
                                flash_success("Type '%s' updated successfully!" % post_type.name)
                except Exception as ex:
                    flash_error("Error: %s" % ex.message)
                return redirect(url_for("CmsAdmin:types"))
            else:
                types = PostModel.Type.all().order_by(PostModel.Type.name.asc())
                return self.render(types=types,
                                   view_template=template_page % "types")

        @nav_menu("Images", endpoint="CmsAdmin:images", order=6, **nav_menu_context)
        @route("%s/images" % route_base, methods=["GET", "POST"], endpoint="CmsAdmin:images")
        def cms_admin_images(self):
            self.page_meta(title="Images")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                description = request.form.get("description")
                if id:
                    image = PostModel.UploadObject.get(id)
                    if image:
                        if action == "delete":
                            image.delete()
                            obj = storage.get(image.name)
                            if obj:
                                obj.delete()
                            flash_success("Image deleted successfully!")
                        else:
                            image.update(description=description)
                            flash_success("Image updated successfully!")
                else:
                    abort(404, "No image ID provided")
                return redirect(url_for("CmsAdmin:images"))

            else:
                page = request.args.get("page", 1)
                per_page = self.config("PAGINATION_PER_PAGE", 25)
                images = PostModel.UploadObject.all()\
                    .filter(PostModel.UploadObject.type == "IMAGE")\
                    .order_by(PostModel.UploadObject.name.asc())
                images = images.paginate(page=page, per_page=per_page)
                return self.render(images=images,
                                   view_template=template_page % "images")

        @route("%s/upload-image" % route_base, methods=["POST"], endpoint="CmsAdmin:upload_image")
        def cms_admin_upload_image(self):
            """
            Placeholder for markdown
            """
            try:
                ajax = request.form.get("ajax", False)
                allowed_extensions = ["gif", "png", "jpg", "jpeg"]

                if request.files.get("file"):
                    _file = request.files.get('file')
                    obj = storage.upload(_file,
                                         prefix="cms",
                                         allowed_extensions=allowed_extensions,
                                         public=True)

                    if obj:
                        description = os.path.basename(obj.name)
                        description = description.replace(".%s" % obj.extension, "")
                        description = description.split("__")[0]
                        upload_object = PostModel.UploadObject.create(name=obj.name,
                                                                      provider=obj.provider_name,
                                                                      container=obj.container.name,
                                                                      extension=obj.extension,
                                                                      type=obj.type,
                                                                      object_path=obj.path,
                                                                      object_url=obj.url,
                                                                      size=obj.size,
                                                                      description=description)
                        if ajax:
                            return jsonify({
                                "id": upload_object.id,
                                "url": upload_object.object_url
                            })
                        else:
                            flash_success("Image '%s' uploaded successfully!" % upload_object.name)

                else:
                    flash_error("Error: Upload object file is invalid or doesn't exist")
            except Exception as e:
                flash_error("Error: %s" % e.message)
            return redirect(url_for("CmsAdmin:images"))

    return Admin



"""
WebPortfolio

"""
import os
import datetime
import inspect
import utils
from werkzeug.contrib.fixers import ProxyFix
from werkzeug.routing import BaseConverter
from flask import Flask, render_template, flash, session
from flask_classy import FlaskView, route
from flask_assets import Environment
import jinja2
import warnings
import __about__

# ------------------------------------------------------------------------------


NAME = __about__.name
__version__ = __about__.version
__author__ = __about__.author
__license__ = __about__.license
__copyright__ = __about__.copyright

# ------------------------------------------------------------------------------

__all__ = ["WebPortfolio",
           "get_env",
           "get_env_config",
           "flash_error",
           "flash_success",
           "flash_info",
           "flash_data",
           "get_flashed_data",
           "route"]

class RegexConverter(BaseConverter):
    def __init__(self, url_map, *items):
        super(RegexConverter, self).__init__(url_map)
        self.regex = items[0]

class WebPortfolio(FlaskView):
    """ WebPortfolio """
    LAYOUT = "layout.html"  # The default layout
    PAGE_META = "PAGE_META"
    GLOBAL_KEY_CONTEXT = "__g"
    assets = None
    _app = None
    _bind = set()
    _template_paths = set()
    _static_paths = set()
    _asset_bundles = set()
    _default_page_meta = dict(
            title="",
            description="",
            url="",
            image="",
            site_name="",
            object_type="article",
            locale="",
            keywords=[],
            use_opengraph=True,
            use_googleplus=True,
            use_twitter=True
        )
    _global = dict(
        WEBPORTFOLIO_NAME=NAME,
        WEBPORTFOLIO_VERSION=__version__,
        YEAR=datetime.datetime.now().year,
        PAGE_META=_default_page_meta
    )

    @classmethod
    def init(cls, flask_or_import_name, project=None, directory=None, config=None,
             compress_html=True):
        """
        Allow to register all subclasses of WebPortfolio at once

        If a class doesn't have a route base, it will create a dasherize version
        of the class name.

        So we call it once initiating
        :param flask_or_import_name: Flask instance or import name -> __name__
        :param project: name of the project. If the directory and config is empty, it will guess them from here
        :param directory: The directory containing your project's Views, Templates and Static
        :param config: string of config object. ie: "app.config.Dev"
        :param compress_html: bool - If true it will use the plugin "jinja2htmlcompress"
                to remove white spaces off the html resul
        """

        if isinstance(flask_or_import_name, Flask):
            app = flask_or_import_name
        else:
            app = Flask(flask_or_import_name)

        app.wsgi_app = ProxyFix(app.wsgi_app)

        app.url_map.converters['regex'] = RegexConverter

        if not directory:
            directory = "application/%s" % project if project else "."

        if not config:
            config = "application.config.%s" % get_env()

        app.config.from_object(config)

        if directory:
            app.template_folder = directory + "/templates"
            app.static_folder = directory + "/static"

        cls.add_asset_bundle(app.static_folder)

        # Extensions to remove extra white spaces in html
        if compress_html:
            app.jinja_env.add_extension('webportfolio.jinja2htmlcompress.HTMLCompress')

        cls._app = app

        # Flask Assets
        cls.assets = Environment(cls._app)

        # Register templates
        if cls._template_paths:
            loader = [cls._app.jinja_loader] + list(cls._template_paths)
            cls._app.jinja_loader = jinja2.ChoiceLoader(loader)

        # Register static
        if cls._static_paths:
            loader = [cls._app.static_folder] + list(cls._static_paths)
            cls.assets.load_path = loader

        # init_app
        for _app in cls._bind:
            _app(cls._app)

        # Register all views
        for subcls in cls.__subclasses__():
            route_base = subcls.route_base
            if not route_base:
                route_base = utils.dasherize(utils.underscore(subcls.__name__))
            subcls.register(cls._app, route_base=route_base)

        # Load all bundles
        for a in cls._asset_bundles:
            cls.assets.from_yaml(a)

        @cls._app.after_request
        def _before_request_cleanup(response):
            cls._global["PAGE_META"] = cls._default_page_meta
            return response

        return cls._app

    @classmethod
    def bind(cls, kls):
        """
        To bind middlewares that needs the 'app' object to init
        Bound middlewares will be assigned on cls.init()
        """
        if not hasattr(kls, "__call__"):
            raise TypeError("From WebPortfolio.bind: '%s' is not callable" % kls)
        cls._bind.add(kls)
        return kls

    @classmethod
    def render(cls, data={}, view_template=None, layout=None, **kwargs):
        """
        To render data to the associate template file of the action view
        :param data: The context data to pass to the template
        :param view_template: The file template to use. By default it will map the classname/action.html
        :param layout: The body layout, must contain {% include __template__ %}
        """
        if not view_template:
            stack = inspect.stack()[1]
            module = inspect.getmodule(cls).__name__
            module_name = module.split(".")[-1]
            action_name = stack[3]      # The method being called in the class
            view_name = cls.__name__    # The name of the class without View

            if view_name.endswith("View"):
                view_name = view_name[:-4]
            view_template = "%s/%s.html" % (view_name, action_name)

        data = data or dict()
        data["__g"] = cls._global
        if kwargs:
            data.update(kwargs)

        data["__template__"] = view_template

        return render_template(layout or cls.LAYOUT, **data)

    @classmethod
    def g(cls, **kwargs):
        """
        Assign a global view context to be used in the template
        :params **kwargs:
        """
        cls._global.update(kwargs)
        
    @classmethod
    def config(cls, key, default=None):
        """
        Shortcut to access the config in your class
        :param key: The key to access
        :param default: The default value when None
        :returns mixed:
        """
        return cls._app.config.get(key, default)

    @classmethod
    def page_meta(cls, **kwargs):
        """
        Meta allows you to add meta data to site
        :params **kwargs:

        meta keys we're expecting:
            title (str)
            description (str)
            url (str) (Will pick it up by itself if not set)
            image (str)
            site_name (str) (but can pick it up from config file)
            object_type (str)
            keywords (list)
            locale (str)
            card (str)

            **Boolean By default these keys are True
            use_opengraph
            use_twitter
            use_googleplus

        """
        n = cls.PAGE_META
        page_meta = cls._global.get(n, {})
        page_meta.update(**kwargs)
        cls.g(n=page_meta)

    @classmethod
    def register_module(cls, root_pkg=None, template="templates", static="static"):
        """
        Register a module's template directory and static
        :param root_pkg: str - The root dir,
                        or the dotted resource package (package.path.path,
                        usually __name__ of templates and static
        :param template: str - The template dir name. If root_dir is none,
                        template should be a path
        :param static: str - The static dir name. If root_dir is none,
                        static should be a path
        """
        if root_pkg:
            if not os.path.isdir(root_pkg) and "." in root_pkg:
                root_pkg = utils.get_pkg_resources_filename(root_pkg)

            template_path = os.path.join(root_pkg, template)
            static_path = os.path.join(root_pkg, static)
        else:
            template_path = template
            static_path = static

        if os.path.isdir(template_path):
            template_path = jinja2.FileSystemLoader(template_path)
            cls._template_paths.add(template_path)
        else:
            warnings.warn("Module registration: Not a <template> directory '%s' " % template_path)

        if os.path.isdir(static_path):
            cls._static_paths.add(static_path)
            cls.add_asset_bundle(static_path)
        else:
            warnings.warn("Module registration: Not a <static> directory '%s' " % static_path)

    @classmethod
    def add_asset_bundle(cls, path):
        """
        Add a webassets bundle yml file
        """
        f = "%s/assets.yml" % path
        if os.path.isfile(f):
            cls._asset_bundles.add(f)

    @classmethod
    def extends(cls, kls):
        """
        A view decorator to extend another view class or function to itself
        It will inherit all its methods and propeties and use them on itself

        -- EXAMPLES --

        class Index(WebPortfolio):
            pass

        index = Index()

        ::-> As decorator on classes ::
        @index.extends__
        class A(object):
            def hello(self):
                pass

        @index.extends__
        class C()
            def world(self):
                pass

        ::-> Decorator With function call ::
        @index.extends__
        def hello(self):
            pass

        """
        if inspect.isclass(kls):
            for _name, _val in kls.__dict__.items():
                if not _name.startswith("__"):
                    setattr(cls, _name, _val)
        elif inspect.isfunction(kls):
            setattr(cls, kls.__name__, kls)
        return cls

def get_env():
    """
    Return the Capitalize environment name
    It can be used to retrieve class base config
    Default: Development
    :returns: str
    """
    env_key = "WEBPORTFOLIO_ENV"
    env = "Development"
    if env_key in os.environ:
        env = os.environ[env_key].lower().capitalize()
    return env

def get_env_config(config):
    """
    Return config class
    :param config : Object - The configuration module containing the environment object
    """
    return getattr(config, get_env())

# Flashing
def flash_error(message):
    """ Set an error message """
    flash(message, "error")

def flash_success(message):
    """ Set an success message """
    flash(message, "success")

def flash_info(message):
    """ Set an info message """
    flash(message, "info")

def flash_data(data):
    """
    Just like flash, but will save data
    :param data:
    :return:
    """
    session["_flash_data"] = data

def get_flashed_data():
    """
    Retrieved
    :return: mixed
    """
    return session.pop("_flash_data", None)



"""
WebPortfolio
"""

import datetime
from flask import redirect, request, url_for, session, jsonify
from webportfolio import (WebPortfolio, route, extends, nav_menu,
                       mailer, cache, storage, recaptcha, csrf,
                       with_user_roles,
                       flash_error, flash_success, flash_info, flash_data, get_flashed_data,
                       ModelError, ViewError)

from webportfolio.module import contact_page, user, cms
from application import model

# ------------------------------------------------------------------------------
# ADMIN ------------------------------------------------------------------------

# ADMIN
# The admin view
@nav_menu("Admin", group="admin")
@extends(cms.admin, model=model)
@extends(user.admin, model=model)
class Admin(WebPortfolio):
    LAYOUT = "admin-layout.html"
    route_base = "admin"
    decorators = [with_user_roles('superadmin', 'admin')]

    @nav_menu("Home")
    def index(self):
        self.page_meta(title="My Admin Home Page")
        return self.render()

    @nav_menu("Page 2")
    def page_2(self):
        self.page_meta(title="My 2nd Page")
        return self.render()

# ------------------------------------------------------------------------------
# Account
@nav_menu("My Account", group="account", order=3, pull_right=True)
@extends(user.account, model=model)
class Account(WebPortfolio):

    @nav_menu("My Account", order=1)
    def index(self):
        self.page_meta(title="My Account")
        return self.render()

    @nav_menu("Upload Image Demo", order=2)
    @route("upload", methods=["GET", "POST"])
    def upload(self):

        self.page_meta(title="Upload Demo")

        if request.method == "POST":
            try:
                _file = request.files.get('file')
                if _file:
                    my_object = storage.upload(_file,
                                               prefix="demo/",
                                               public=True,
                                               allowed_extensions=["gif", "jpg", "jpeg", "png"])
                    if my_object:
                        return redirect(url_for("Account:upload", object_name=my_object.name))
            except Exception as e:
                flash_error(e.message)
            return redirect(url_for("Account:upload"))

        my_object = None
        object_name = request.args.get("object_name")
        if object_name:
            my_object = storage.get(object_name=object_name)

        return self.render(my_object=my_object)


# ------------------------------------------------------------------------------
# Blog
@extends(cms.post,
         model=model,
         query={
             "types": ["blog"],
             "order_by": "published_at desc",
         },
         endpoints={
             "index": {"menu": "Blog", "endpoint": "/", "per_page": 10,
                       "post_title": "My Blog",
                       "post_header": "My Blog",
                       "post_subheader": ""},
             "single": {"menu": "Read", "endpoint": "/:slug"},
             "authors": {"menu": "Authors", "endpoint": "authors", "show_menu": False},
             "archive": {"menu": "Archive", "endpoint": "archive", "show_menu": False},
         }
         )
class Blog(WebPortfolio):
    pass

# ------------------------------------------------------------------------------
# Default
#@nav_menu("Home", key="Main", order=1)
@extends(cms.post, model=model,
         query={
             "types": ["page"],
             "order_by": "title asc"
         },
         endpoints={
             "index": {"menu": "Pages",
                       "endpoint": "pages",
                       "post_title": "Pages",
                       "post_show_byline": False},
             "single": {"menu": "Documents", "endpoint": "page/:slug", "post_show_byline": False},
             "archive": {"menu": "Archive", "endpoint": "archive", "show_menu": False},
             "authors": {"menu": "Authors", "endpoint": "authors", "show_menu": False}
         }
         )
@extends(contact_page.contact_page)
class Index(WebPortfolio):
    route_base = "/"

    @nav_menu("Home", order=1)
    def index(self):
        self.page_meta(title="Hello WebPortfolio!")
        return self.render()






from webportfolio import WebPortfolio, logger

WebPortfolio.register_module(__name__)

def view(template=None):
    """
    Create the Maintenance view
    Must be instantiated

    import maintenance_view
    MaintenanceView = maintenance_view()

    :param view_template: The directory containing the view pages
    :return:
    """
    if not template:
        template = "WebPortfolio/Module/MaintenancePage/index.html"

    class Maintenance(WebPortfolio):
        @classmethod
        def register(cls, app, **kwargs):
            super(cls, cls).register(app, **kwargs)

            if cls.config("MODULE_MAINTENANCE_PAGE_ON"):
                logger.info("MAINTENANCE PAGE IS ON")

                @app.before_request
                def on_maintenance():
                    return cls.render(layout=template), 503

    return Maintenance

Maintenance = view()


"""
WebPortfolio

Command line tool

web-portfolio -c project_name

"""

import os
import re
import sys
import logging
import importlib
import pkg_resources
import utils
import __about__
import click

CWD = os.getcwd()
SKELETON_DIR = "app-skel"
APPLICATION_DIR = "%s/application" % CWD
APPLICATION_DATA_DIR = "%s/data" % APPLICATION_DIR

def get_project_dir_path(project_name):
    return "%s/%s" % (APPLICATION_DIR, project_name)

def copy_resource(src, dest):
    """
    To copy package data to destination
    """
    dest = (dest + "/" + os.path.basename(src)).rstrip("/")
    if pkg_resources.resource_isdir("webportfolio", src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        for res in pkg_resources.resource_listdir(__name__, src):
            copy_resource(src + "/" + res, dest)
    else:
        if os.path.splitext(src)[1] not in [".pyc"]:
            with open(dest, "wb") as f:
                f.write(pkg_resources.resource_string(__name__, src))


def create_project(project_name, template="app"):
    """
    Create the project
    """

    project_dir = get_project_dir_path(project_name)
    serve_tpl = pkg_resources.resource_string(__name__, '%s/serve.py' % (SKELETON_DIR))
    propel_tpl = pkg_resources.resource_string(__name__, '%s/propel.yml' % (SKELETON_DIR))
    config_tpl = pkg_resources.resource_string(__name__, '%s/config.py' % (SKELETON_DIR))
    model_tpl = pkg_resources.resource_string(__name__, '%s/model.py' % (SKELETON_DIR))
    manage_tpl = pkg_resources.resource_string(__name__, '%s/manage.py' % (SKELETON_DIR))

    serve_file = "%s/serve_%s.py" % (CWD, project_name)
    requirements_txt = "%s/requirements.txt" % CWD
    propel_yml = "%s/propel.yml" % CWD
    config_py = "%s/config.py" % APPLICATION_DIR
    model_py = "%s/model.py" % APPLICATION_DIR
    manage_py = "%s/manage.py" % CWD

    dirs = [
        APPLICATION_DIR,
        APPLICATION_DATA_DIR,
        project_dir
    ]
    for dir in dirs:
        if not os.path.isdir(dir):
            os.makedirs(dir)

    files = [
        ("%s/__init__.py" % APPLICATION_DIR, "# WebPortfolio"),
        (config_py, config_tpl),
        (model_py, model_tpl),
        (serve_file, serve_tpl.format(project_name=project_name)),
        (requirements_txt, "%s==%s" % (__about__.name, __about__.version)),
        (propel_yml, propel_tpl.format(project_name=project_name)),
        (manage_py, manage_tpl)
    ]

    for file in files:
        if not os.path.isfile(file[0]):
            with open(file[0], "wb") as f:
                f.write(file[1])

    copy_resource("%s/%s/" % (SKELETON_DIR, template), project_dir)

    copy_resource("%s/%s/" % (SKELETON_DIR, "data"), APPLICATION_DATA_DIR)

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

def _addCWDToSysPath():
    sys.path.append(CWD)

def _title(title=None):
    _description = "-" * 80
    _description += "\n%s %s" % (__about__.name, __about__.version)
    click.echo(_description)
    click.echo("")
    if title:
        click.echo("** %s **" % title)
        click.echo("")

_object_name_regex = re.compile('[^a-zA-Z]')
def format_project_name(name):
    return _object_name_regex.sub("", name)

def get_project_serve_module(project):
    _addCWDToSysPath()
    if "serve_" not in project:
        project = format_project_name(project)
        project = "serve_%s" % project
    return importlib.import_module(project)

def import_project_module(module):
    _addCWDToSysPath()
    return importlib.import_module(module)

@click.group()
def cli():
    pass

@cli.command()
@click.option("--project", "-p", default="www")
def new(project):
    """  Create a new project """

    project = format_project_name(project)

    _title("Creating new project ...")
    click.echo("- Project: %s " % project)

    create_project(project)

    click.echo("- Sweet! Your new project [ %s ] has been created" % project)
    click.echo("- Location: [ application/%s ]" % project)
    click.echo("> What's next?")
    click.echo("- Edit the config [ application/%s/config.py ] " % project)
    click.echo("- If necessary edit and run the manager [ python manage.py setup ]")
    click.echo("- Launch server, run [ webportfolio serve -p %s ]" % project)
    click.echo("")

@cli.command(name="build-assets")
@click.option("--project", "-p", default="www")
def build_assets(project):
    """
    Build application's assets files
    """

    from webassets.script import CommandLineEnvironment
    _title("Build application's assets files from bundles ...")
    click.echo("- Project: %s " % project)
    click.echo("")

    module = get_project_serve_module(project)
    app = module.app
    assets_env = app.jinja_env.assets_environment

    log = logging.getLogger('webassets')
    log.addHandler(logging.StreamHandler())
    log.setLevel(logging.DEBUG)

    cmdenv = CommandLineEnvironment(assets_env, log)
    cmdenv.build()

@cli.command()
@click.option("--project", "-p", default="www")
def assets2s3(project):
    """ To upload assets to S3"""

    import flask_s3

    _title("Upload assets files to S3 ...")
    click.echo("- Project: %s " % project)
    click.echo("")

    module = get_project_serve_module(project)
    app = module.app
    flask_s3.create_all(app)

@cli.command()
@click.option("--project", "-p", default="www")
def serve(project):
    """ Serve a project in Local Development environment
    """
    _title("Start server in DEV environment ...")
    click.echo("- Project: %s " % project)
    click.echo("")

    module = get_project_serve_module(project)
    app = module.app
    app.run(debug=True)

#@cli.command()
#@click.option("--project", "-p", default="www")
def admin(project):
    app = get_project_serve_module(project).app
    manager = import_project_module("manage").manager()
    manager()



# About

name = "WebPortfolio"
title = "WebPortfolio"
version = "0.0.6"
author = "Mardix"
email = "mardix@pylot.io"
description = "A full stack Flask based framework that put structure in the " \
              "file system. Features: Caching, Mailing, Storage, CSRF, recaptcha " \
              "and more"
uri = "https://github.com/mardix/web-portfolio"
license = "MIT"
copyright = "(c) 2015 %s" % author



from webportfolio import WebPortfolio

WebPortfolio.register_module(__name__)

def view(template_dir=None):
    """
    Create the Error view
    Must be instantiated

    import error_view
    ErrorView = error_view()

    :param template_dir: The directory containing the view pages
    :return:
    """
    if not template_dir:
        template_dir = "WebPortfolio/Module/ErrorPage"

    template_page = "%s/index.html" % template_dir

    class Error(WebPortfolio):
        """
        Error Views
        """
        @classmethod
        def register(cls, app, **kwargs):
            super(cls, cls).register(app, **kwargs)

            @app.errorhandler(400)
            def error_400(error):
                return cls.index(error, 400)

            @app.errorhandler(401)
            def error_401(error):
                return cls.index(error, 401)

            @app.errorhandler(403)
            def error_403(error):
                return cls.index(error, 403)

            @app.errorhandler(404)
            def error_404(error):
                return cls.index(error, 404)

            @app.errorhandler(500)
            def error_500(error):
                return cls.index(error, 500)

            @app.errorhandler(503)
            def error_503(error):
                return cls.index(error, 503)

        @classmethod
        def index(cls, error, code):
            cls.page_meta(title="Error %s" % code)

            return cls.render(error=error, view_template=template_page), code
    return Error

ErrorV = view()


"""
Load and setup extensions
"""
import re
import warnings
import logging
import inspect
import functools
from six.moves.urllib.parse import urlparse
from flask import abort, request, current_app
from core import WebPortfolio
import utils
import humanize
import wp_markdown
import flask_cloudy
import flask_recaptcha
import flask_seasurf
import flask_kvsession
import flask_cache
import flask_login
import ses_mailer
import flask_mail
import flask_s3


__all__ = ["extends",
           "with_user_roles",
           "login_required",
           "no_login_required",
           "nav_menu",
           "mailer",
           "cache",
           "storage",
           "recaptcha",
           "csrf",
           "logger",
           "warn_missing_config"]

# ------------------------------------------------------------------------------
def extends(module, *args, **kwargs):
    """
    Decorator to extend a module to a view.
    The module can be a class or function. It will copy all the methods to the class

    ie:
        #Your module.py
        welcome_module(view, **kwargs):

            @route("/welcome-home")
            my_view():
                pass
            return my_view

        #Your view.py
        @extends(welcome_module)
        class Index(WebPortfolio):
            pass

    :param module: object
    :param args:
    :param kwargs:
    :return:
    """
    def wrap(view):
        return view.extends(module(view, *args, **kwargs))
    return wrap

# ------------------------------------------------------------------------------

# user_*

def user_authenticated():
    """
    A shortcut to check if a user is authenticated
    :return: bool
    """
    return True if flask_login.current_user and flask_login.current_user.is_authenticated else False

def user_not_authenticated():
    """
    A shortcut to check if user not authenticated.
    """
    return not user_authenticated()

def with_user_roles(*roles):
    """
    A decorator to check if user has any of the roles specified

    @with_user_roles('superadmin', 'admin')
    def fn():
        pass
    """
    def wrapper(f):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            if user_authenticated():
                if not flask_login.current_user.has_any_roles(*roles):
                    return abort(403)
            else:
                return abort(401)
            return f(*args, **kwargs)
        return wrapped
    return wrapper

def with_user_permissions(*permissions):
    pass


def login_required(func):
    """
    A wrapper around the flask_login.login_required.
    But it also checks the presence of the decorator: @no_login_required
    On a "@login_required" class, method containing "@no_login_required" will
    still be able to access without authentication
    :param func:
    :return:
    """
    @functools.wraps(func)
    def decorated_view(*args, **kwargs):
        if "no_login_required" not in utils.get_decorators_list(func) \
                and user_not_authenticated():
            return current_app.login_manager.unauthorized()
        return func(*args, **kwargs)
    return decorated_view

def no_login_required(func):
    """
    Dummy decorator. @login_required will inspect the method
    to look for this decorator
    Use this decorator when you want do not require login in a "@login_required" class/method
    :param func:
    :return:
    """
    @functools.wraps(func)
    def decorated_view(*args, **kwargs):
        return func(*args, **kwargs)
    return decorated_view

# ------------------------------------------------------------------------------
#
# @nav_menu
#
# Decorator to build navigation menu directly from the methods
# By default it will build the menu of the module, class an method
# If the class is also decorated, it will use the menu name as the top
# level name

_NAV_MENU_STACK = {}

def nav_menu(name, **kwargs):
    """
    Decorator to build navigation menu directly on the methods
    By default it will build the menu of the module, class an method
    If the class is also decorated, it will use the menu name as the top level name

    :param name: The menu name
    :param kwargs: extra options to pass into the menu or to move the menu somewhere else

        order int: The order of the menu in the set

        show (list of bool or callback): To hide and show menu. Accepts bool or
                    list of callback function the callback function must return
                    a bool to check if all everything is True to show or will be False
                    ** When this menu is inside of a menu set, or has parent, if you want
                    that page to be activated, but don't want to create a menu link,
                    for example: a blog read page, set show to False. It will know
                    the menu set is active

        endpoint string: By default the endpoint is built based on the method and class.
                    When set it will be used instead

        group: On class menu, it can be used to filter a menu set to display

        The args below will allow you to change where the menu is placed.
        By default they are set automatically

        __module: the module name. Usually if using another module
        __class: the class name class name in the module
        __method: The method name, to build endpoint. Changing this will cha
        __is_class: When the decorator is on the class instead of the method.
            This will be used to make the top level.
            Function not inside of class, may say it's a class, set __is_class = False

    :return:
    """
    def wrap(f):
        if name is not None:
            __module = kwargs.pop("__module", f.__module__)
            __class = kwargs.pop("__class", inspect.stack()[1][3])
            __is_class = kwargs.pop("__is_class", inspect.isclass(f))
            __method = kwargs.pop("__method", f.__name__)

            _push_nav_menu(__module=__module,
                           __class=__class,
                           __method=__method,
                           __is_class=__is_class,
                           name=name,
                           **kwargs)

        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            return f(*args, **kwargs)
        return wrapper
    return wrap

def _push_nav_menu(**kwargs):

    __module = kwargs.pop("__module")
    __class = kwargs.pop("__class")
    __method = kwargs.pop("__method")
    __is_class = kwargs.pop("__is_class")

    __cls = __method if __is_class else __class
    path = "%s.%s" % (__module, __cls)

    if path not in _NAV_MENU_STACK:
        _NAV_MENU_STACK[path] = {
            "name": None,
            "endpoint": None,
            "order": None,
            "sub_menu": [],
            "kwargs": {}
        }

    if "name" not in kwargs:
        raise TypeError("Missing 'name' in menu decorator")

    kwargs["endpoint"] = kwargs.pop("endpoint", "%s:%s" % (__cls,
                                                         "index" if __is_class else __method ))
    order = kwargs.pop("order", 0)
    name = kwargs.pop("name")
    endpoint = kwargs.pop("endpoint")

    # show: accepts a bool or list of callback to execute
    show = kwargs.pop("show", [])

    if isinstance(show, bool):
        show = [show]
    elif show and not isinstance(show, list):
        show = [show]

    kwargs["show"] = show
    kwargs["__active"] = None
    kwargs["__index"] = None

    if __is_class:  # class menu
        kwargs["endpoint"] = endpoint
        kwargs["group"] = kwargs.pop("group", None)
        _NAV_MENU_STACK[path]["name"] = name
        _NAV_MENU_STACK[path]["order"] = order
        _NAV_MENU_STACK[path]["kwargs"] = kwargs
    else:  # sub menu
        menu = (order, name, endpoint, kwargs)
        _NAV_MENU_STACK[path]["sub_menu"].append(menu)

def _nav_menu_test_show(shows):
    return all([x() if hasattr(x, "__call__") else x for x in shows])

def _nav_menu_init(app):
    @app.before_request
    def p(*args, **kwargs):

        # (order, Name, sub_menu, **kargs, hidden_submenu)
        menu_list = []
        menu_index = 0
        for _, menu in _NAV_MENU_STACK.items():
            menu_index += 1

            menu["kwargs"]["__index"] = menu_index
            menu["kwargs"]["__active"] = False

            # Filter out menu that should not be shown
            _kw = menu["kwargs"]
            if _kw and "show" in _kw and _nav_menu_test_show(_kw["show"]) is False:
                continue

            sub_menu = []
            sub_menu_hidden = []  # Save all the hidden submenu

            for s in menu["sub_menu"]:
                if s[2] == request.endpoint:
                    s[3]["__active"] = True
                    menu["kwargs"]["__active"] = True

                sub_menu_hidden.append(s)

                _kw = s[3]
                if _kw and "show" in _kw and _nav_menu_test_show(_kw["show"]) is False:
                    continue
                sub_menu.append(s)

            _kwargs = menu["kwargs"]
            _kwargs["__hidden"] = sorted(sub_menu_hidden)
            menu_list.append((
                menu["order"],
                menu["name"],
                sorted(sub_menu),
                _kwargs
            ))

        WebPortfolio.g(NAV_MENU=sorted(menu_list))

WebPortfolio.bind(_nav_menu_init)

# ------------------------------------------------------------------------------

# Session
#
# It uses KV session to allow multiple backend for the session
def _session(app):
    store = None
    uri = app.config.get("SESSION_URI")
    if uri:
        parse_uri = urlparse(uri)
        scheme = parse_uri.scheme
        username = parse_uri.username
        password = parse_uri.password
        hostname = parse_uri.hostname
        port = parse_uri.port
        bucket = parse_uri.path.strip("/")

        if "redis" in scheme:
            import redis
            from simplekv.memory.redisstore import RedisStore
            conn = redis.StrictRedis.from_url(url=uri)
            store = RedisStore(conn)
        elif "s3" in scheme or "google_storage" in scheme:
            from simplekv.net.botostore import BotoStore
            import boto
            if "s3" in scheme:
                _con_fn = boto.connect_s3
            else:
                _con_fn = boto.connect_gs
            conn = _con_fn(username, password)
            _bucket = conn.create_bucket(bucket)
            store = BotoStore(_bucket)
        elif "memcache" in scheme:
            import memcache
            from simplekv.memory.memcachestore import MemcacheStore
            host_port = "%s:%s" % (hostname, port)
            conn = memcache.Client(servers=[host_port])
            store = MemcacheStore(conn)
        elif "sql" in scheme:
            from simplekv.db.sql import SQLAlchemyStore
            from sqlalchemy import create_engine, MetaData
            engine = create_engine(uri)
            metadata = MetaData(bind=engine)
            store = SQLAlchemyStore(engine, metadata, 'kvstore')
            metadata.create_all()
        else:
            raise ValueError("Invalid Session Store")
    if store:
        flask_kvsession.KVSessionExtension(store, app)

WebPortfolio.bind(_session)

# ------------------------------------------------------------------------------
# Mailer
class _Mailer(object):
    """
    A simple wrapper to switch between SES-Mailer and Flask-Mail based on config
    """
    mail = None
    provider = None
    config = None
    _template = None

    def init_app(self, app):
        self.config = app.config
        scheme = None

        mailer_uri = self.config.get("MAILER_URI")
        if mailer_uri:
            mailer_uri = urlparse(mailer_uri)
            scheme = mailer_uri.scheme

            # Using ses-mailer
            if "ses" in scheme:
                self.provider = "SES"

                access_key = mailer_uri.username or app.config.get("AWS_ACCESS_KEY_ID")
                secret_key = mailer_uri.password or app.config.get("AWS_SECRET_ACCESS_KEY")

                self.mail = ses_mailer.Mail(aws_access_key_id=access_key,
                                            aws_secret_access_key=secret_key,
                                            sender=self.config.get("MAILER_SENDER"),
                                            reply_to=self.config.get("MAILER_REPLY_TO"),
                                            template=self.config.get("MAILER_TEMPLATE"),
                                            template_context=self.config.get("MAILER_TEMPLATE_CONTEXT"))

            # SMTP will use flask-mail
            elif "smtp" in scheme:
                self.provider = "SMTP"

                class _App(object):
                    config = {
                        "MAIL_SERVER": mailer_uri.hostname,
                        "MAIL_USERNAME": mailer_uri.username,
                        "MAIL_PASSWORD": mailer_uri.password,
                        "MAIL_PORT": mailer_uri.port,
                        "MAIL_USE_TLS": True if "tls" in mailer_uri.scheme else False,
                        "MAIL_USE_SSL": True if "ssl" in mailer_uri.scheme else False,
                        "MAIL_DEFAULT_SENDER": app.config.get("MAILER_SENDER"),
                        "TESTING": app.config.get("TESTING"),
                        "DEBUG": app.config.get("DEBUG")
                    }
                    debug = app.config.get("DEBUG")
                    testing = app.config.get("TESTING")

                _app = _App()
                self.mail = flask_mail.Mail(app=_app)

                _ses_mailer = ses_mailer.Mail(template=self.config.get("MAILER_TEMPLATE"),
                                              template_context=self.config.get("MAILER_TEMPLATE_CONTEXT"))
                self._template = _ses_mailer.parse_template
            else:
                warnings.warn("Mailer Error. Invalid scheme '%s'>" % scheme)

    def send(self, to, subject, body, reply_to=None, **kwargs):
        """
        Send simple message
        """
        if self.provider == "SES":
            self.mail.send(to=to,
                           subject=subject,
                           body=body,
                           reply_to=reply_to,
                           **kwargs)

        elif self.provider == "SMTP":
            msg = flask_mail.Message(recipients=[to] if not isinstance(to, list) else to,
                                     subject=subject,
                                     body=body,
                                     reply_to=reply_to,
                                     sender=self.config.get("MAILER_SENDER"))
            self.mail.send(msg)
        else:
            abort(500, "Mailer Error. Invalid 'provider'")

    def send_template(self, template, to, reply_to=None, **context):
        """
        Send Template message
        """
        if self.provider == "SES":
            self.mail.send_template(template=template,
                                    to=to,
                                    reply_to=reply_to,
                                    **context)

        elif self.provider == "SMTP":
            data = self._template(template=template, **context)
            msg = flask_mail.Message(recipients=[to] if not isinstance(to, list) else to,
                                     subject=data["subject"],
                                     body=data["body"],
                                     reply_to=reply_to,
                                     sender=self.config.get("MAILER_SENDER"))
            self.mail.send(msg)
        else:
            abort(500, "Mailer Error. Invalid 'provider'")

mailer = _Mailer()
WebPortfolio.bind(mailer.init_app)

# ------------------------------------------------------------------------------

# Assets Delivery
class _AssetsDelivery(flask_s3.FlaskS3):

    def init_app(self, app):
        delivery_method = app.config.get("ASSETS_DELIVERY_METHOD")
        if delivery_method and delivery_method.upper() in ["S3", "CDN"]:
            #with app.app_context():
            is_secure = False #request.is_secure

            if delivery_method.upper() == "CDN":
                domain = app.config.get("ASSETS_DELIVERY_DOMAIN")
                if "://" in domain:
                    domain_parsed = urlparse(domain)
                    is_secure = domain_parsed.scheme == "https"
                    domain = domain_parsed.netloc
                app.config.setdefault("S3_CDN_DOMAIN", domain)

            app.config["FLASK_ASSETS_USE_S3"] = True
            app.config["USE_S3"] = True
            app.config.setdefault("S3_USE_HTTPS", is_secure)
            app.config["S3_URL_STYLE"] = "path"
            app.config.setdefault("S3_ONLY_MODIFIED", False)
            app.config.setdefault("S3_BUCKET_NAME", app.config.get("AWS_S3_BUCKET_NAME"))

            super(self.__class__, self).init_app(app)

assets_delivery = _AssetsDelivery()
WebPortfolio.bind(assets_delivery.init_app)

# ------------------------------------------------------------------------------

# Cache
cache = flask_cache.Cache()
WebPortfolio.bind(cache.init_app)

# Storage
storage = flask_cloudy.Storage()
WebPortfolio.bind(storage.init_app)

# Recaptcha
recaptcha = flask_recaptcha.ReCaptcha()
WebPortfolio.bind(recaptcha.init_app)

# CSRF
csrf = flask_seasurf.SeaSurf()
WebPortfolio.bind(csrf.init_app)

# LOGGER
logger = logging

def warn_missing_config(conf, k):
    if k not in conf or not conf.get(k) or conf.get(k).strip() == "":
        msg = "Config [ %s ] value is not set or empty" % k
        logger.warn(msg)
        return msg
    return None

# ------------------------------------------------------------------------------

def _setup(app):

    def sanity_check():
        keys = [
            "APPLICATION_ADMIN_EMAIL",
            "MAILER_URI",
            "MAILER_SENDER",
            "MODULE_CONTACT_PAGE_EMAIL",
            "RECAPTCHA_SITE_KEY",
            "RECAPTCHA_SITE_SECRET"
        ]
        [warn_missing_config(app.config, k) for k in keys]

    sanity_check()

    WebPortfolio.g(APPLICATION_NAME=app.config.get("APPLICATION_NAME"),
                   APPLICATION_VERSION=app.config.get("APPLICATION_VERSION"),
                   APPLICATION_URL=app.config.get("APPLICATION_URL"),
                   APPLICATION_GOOGLE_ANALYTICS_ID=app.config.get("APPLICATION_GOOGLE_ANALYTICS_ID"))

    # Setup filters
    @app.template_filter('datetime')
    def format_datetime(dt, format="%m/%d/%Y"):
        return "" if not dt else dt.strftime(format)

    @app.template_filter('strip_decimal')
    def strip_decimal(amount):
        return amount.split(".")[0]

    @app.template_filter('bool_to_yes')
    def bool_to_yes(b):
        return "Yes" if b else "No"

    @app.template_filter('bool_to_int')
    def bool_to_int(b):
        return 1 if b else 0

    @app.template_filter('nl2br')
    def nl2br(s):
        """
        {{ s|nl2br }}

        Convert newlines into <p> and <br />s.
        """
        if not isinstance(s, basestring):
            s = str(s)
        s = re.sub(r'\r\n|\r|\n', '\n', s)
        paragraphs = re.split('\n{2,}', s)
        paragraphs = ['<p>%s</p>' % p.strip().replace('\n', '<br />') for p in paragraphs]
        return '\n\n'.join(paragraphs)

    # More filters
    app.jinja_env.filters.update({
        # slug
        "slug": utils.slugify,
        # Transform an int to comma
        "int_with_comma": humanize.intcomma,
        # Show the date ago: Today, yesterday, July 27 (without year in same year), July 15 2014
        "date_since": humanize.naturaldate,
        # To show the time ago: 3 min ago, 2 days ago, 1 year 7 days ago
        "time_since": humanize.naturaltime,
        # Return a mardown to HTML
        "markdown": wp_markdown.html,
        # Create a Table of Content of the Markdown
        "markdown_toc": wp_markdown.toc
    })

WebPortfolio.bind(_setup)





"""
Web-Portfolio

Web-Portfolio is a Flask based framework to help quickly develop web applications, by
adding structure to your views and templates.

Philosophy:

To create a framework that runs everywhere, regardless of the platform, by
providing cloud balh...

It made the following decisions for you: (of course you can change them)


It comes with pre-built components:


And it is still Flask.



Portfolio is a framework based on Flask extension that adds structure to both your views and
templates, by mapping them to each other to provide a rapid application development framework.
The extension also comes with Flask-Classy, Flask-Assets, Flask-Mail,
JQuery 2.x, Bootstrap 3.x, Font-Awesome, Bootswatch templates.
The extension also provides pre-made templates for error pages and macros.

http://pylot.io/
https://github.com/mardix/web-portfolio

"""

import os
from setuptools import setup, find_packages

base_dir = os.path.dirname(__file__)

__about__ = {}
with open(os.path.join(base_dir, "webportfolio", "__about__.py")) as f:
    exec(f.read(), __about__)

setup(
    name=__about__["title"],
    version=__about__["version"],
    license=__about__["license"],
    author=__about__["author"],
    author_email=__about__["email"],
    description=__about__["description"],
    url=__about__["uri"],
    long_description=__doc__,
    py_modules=['webportfolio'],
    entry_points=dict(console_scripts=[
        'webportfolio=webportfolio.cli:cli',
        'wp=webportfolio.cli:cli',
    ]),
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'Flask==0.10.1',
        'Flask-Classy==0.6.10',
        'Flask-Assets==0.10',
        'flask-recaptcha==0.4.1',
        'flask-login==0.3.0',
        'flask-kvsession==0.6.1',
        'flask-s3==0.1.7',
        'flask-mail==0.9.0',
        'flask-cache==0.13.1',
        'flask-cloudy==0.11.0',
        'flask-seasurf==0.2.0',

        'Active-Alchemy==0.4.4',
        'Paginator==0.3.5',
        'authomatic==0.1.0.post1',
        'six==1.9.0',
        'passlib==1.6.2',
        'bcrypt==1.1.1',
        'python-slugify==0.1.0',
        'humanize==0.5.1',
        'redis==2.9.1',
        'ses-mailer==0.12.1',
        'markdown==2.6.2',
        'inflection==0.3.1',
        'pyyaml==3.11',

        'click==5.1'
    ],
    dependency_links=[
        'https://github.com/maxcountryman/flask-login/archive/4d439e8050a8ee942cb76a0309faec9202b8aca9.zip#egg=flask-login-0.3.0'
    ],
    keywords=['flask',
              'portfolio',
              'templates',
              'views',
              'classy',
              'framework',
              "mvc",
              "blueprint"],
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe=False
)




"""
User Account
"""

import datetime
from flask import url_for, request, redirect, abort, session, jsonify, make_response
from webportfolio import (WebPortfolio, route, mailer, recaptcha, storage, nav_menu,
                       flash_error, flash_success, flash_data, get_flashed_data,
                       login_required, no_login_required,
                       ModelError, ViewError)
from webportfolio.ext import user_authenticated, user_not_authenticated

import webportfolio.utils as utils
from flask_login import (LoginManager, login_user, logout_user, current_user,
                         fresh_login_required, UserMixin)

from authomatic import Authomatic, provider_id
from authomatic.providers import oauth1, oauth2
from authomatic.adapters import WerkzeugAdapter

# Primary Roles
PRIMARY_ROLES = [(99, "SUPERADMIN"),
                 (98, "ADMIN"),
                 (1, "USER")]

class OAuthomatic(object):
    oauth = None
    response = None

    def init_app(self, app):

        if app.config.get("MODULE_USER_ACCOUNT_ENABLE_OAUTH_LOGIN"):
            secret = app.config.get("SECRET_KEY")
            providers = app.config.get("MODULE_USER_ACCOUNT_OAUTH_PROVIDERS")
            config = {}
            auth_providers = []

            for provider, conf in providers.items():
                if hasattr(oauth2, provider):
                    cls = getattr(oauth2, provider)
                    conf["class_"] = conf["class_"] if "class_" in conf else cls
                elif hasattr(oauth1, provider):
                    cls = getattr(oauth1, provider)
                    conf["class_"] = conf["class_"] if "class_" in conf else cls
                else:
                    continue

                conf["id"] = provider_id()
                _provider = provider.lower()
                auth_providers.append(_provider)
                config[_provider] = conf

            self.oauth = Authomatic(
                config=config,
                secret=secret,
                session=session,
                report_errors=True
            )

            WebPortfolio.g(OAUTH_PROVIDERS=auth_providers)

    def login(self, provider):
        response = make_response()
        adapter = WerkzeugAdapter(request, response)
        login = self.oauth.login(adapter=adapter,
                                 provider_name=provider,
                                 session=session,
                                 session_saver=self._session_saver)
        self.response = response
        return login

    def _session_saver(self):
        session.modified = True

WebPortfolio.register_module(__name__)

oauth = OAuthomatic()
WebPortfolio.bind(oauth.init_app)

# ------------------------------------------------------------------------------

# The user_model create a fully built model with social signin
def model(db):

    class UserRole(db.Model):

        name = db.Column(db.String(75), index=True)
        level = db.Column(db.Integer, index=True)

        @classmethod
        def new(cls, name, level):
            name = utils.slugify(name)
            role = cls.get_by_name(name)
            if not role:
                role = cls.create(name=name, level=level)
            return role

        @classmethod
        def get_by_name(cls, name):
            name = utils.slugify(name)
            return cls.all().filter(cls.name == name).first()

        @classmethod
        def get_by_level(cls, level):
            return cls.all().filter(cls.level == level).first()

    class User(UserMixin, db.Model):

        email = db.Column(db.String(75), index=True, unique=True)
        email_confirmed = db.Column(db.Boolean, default=False)
        password_hash = db.Column(db.String(250))
        has_temp_login = db.Column(db.Boolean, default=False)
        temp_login_token = db.Column(db.String(100), index=True)
        temp_login_expiration = db.Column(db.DateTime)
        first_name = db.Column(db.String(250))
        last_name = db.Column(db.String(250))
        date_of_birth = db.Column(db.Date)
        sex = db.Column(db.String(10))   # To get confusion out of the way, Sex refers to natural/biological features.
        profile_image_url = db.Column(db.String(250))
        signup_method = db.Column(db.String(250))
        active = db.Column(db.Boolean, default=True, index=True)
        last_login = db.Column(db.DateTime)
        last_visited = db.Column(db.DateTime)
        roles = db.relationship(UserRole, secondary="user_role_map")

        # ------ FLASK-LOGIN REQUIRED METHODS ----------------------------------

        @property
        def is_active(self):
            return self.active

        # ---------- END FLASK-LOGIN REQUIREMENTS ------------------------------

        @classmethod
        def get_by_email(cls, email):
            """
            Return a User by email address
            """
            return cls.all().filter(cls.email == email).first()

        @classmethod
        def get_by_temp_login(cls, token):
            """
            Return a User by temp_login_token
            temp_login_token allows a user to login with the token
            and reset the password
            """
            user = cls.all().filter(cls.temp_login_token == token).first()
            if user:
                now = datetime.datetime.now()
                if user.has_temp_login is True \
                        and user.temp_login_expiration > now:
                    return user
                user.clear_temp_login()
            return None

        @classmethod
        def get_by_oauth(cls, provider, provider_user_id):
            """
            Get a user by OAuth
            :param provider:
            :param provider_user_id:
            :return: User
            """
            oauth = UserOauthLogin.get_by_provider(provider=provider,
                                                   provider_user_id=provider_user_id)
            return oauth.user if oauth else None
        
        @classmethod
        def new(cls,
                email,
                password=None,
                first_name=None,
                last_name=None,
                role="USER",
                signup_method="email",
                profile_image_url=None,
                **kwargs):
            """
            Create a new user account
            """
            user = cls.get_by_email(email)
            if user:
                raise ModelError("User exists already")
            user = cls.create(email=email,
                              first_name=first_name,
                              last_name=last_name,
                              signup_method=signup_method,
                              profile_image_url=profile_image_url)
            if password:
                user.set_password(password)
            if role:
                role_ = UserRole.get_by_name(role.upper())
                if role_:
                    user.update_roles([role_.id])

            return user

        @property
        def full_name(self):
            """
            Return the full name
            :return:
            """
            return "%s %s" % (self.first_name, self.last_name)

        @property
        def name(self):
            """
            Alias to first_name
            :return:
            """
            return self.first_name

        def password_matched(self, password):
            """
            Check if the password matched the hash
            :returns bool:
            """
            return utils.verify_encrypted_string(password, self.password_hash)

        def set_password(self, password, random=False):
            """
            Encrypt the password and save it in the DB
            Return the password passed or the new password if randomed
            """
            if random:
                password = utils.generate_random_string()
            self.update(password_hash=utils.encrypt_string(password))
            return password

        def set_temp_login(self, expiration=60):
            """
            Create temp login.
            It will allow to have change password on account
            :param expiration: in minutes the time for expiration
            """
            expiration = datetime.datetime.now() + datetime.timedelta(minutes=expiration)
            while True:
                token = utils.generate_random_string(32).lower()
                if not User.all().filter(User.temp_login_token == token).first():
                    break
            self.update(has_temp_login=True,
                        temp_login_token=token,
                        temp_login_expiration=expiration)
            return token

        def clear_temp_login(self):
            self.update(has_temp_login=False,
                        temp_login_token=None,
                        temp_login_expiration=None)

        def update_roles(self, roles_list):
            """
            Update the roles of a user
            :param roles_list: a list of all new roles, it will remove roles not listed
            :return:
            """
            r_list = [r.id for r in self.roles]
            del_roles = list(set(r_list) - set(roles_list))
            new_roles = list(set(roles_list) - set(r_list))
            for dc in del_roles:
                UserRoleMap.remove(user_id=self.id, role_id=dc)
            for nc in new_roles:
                UserRoleMap.add(user_id=self.id, role_id=nc)

        def add_oauth(self, provider, provider_user_id, **kwargs):
            """
            To attach a user account to an OAUTH login
            :param provider: the name of the provider
            :param provider_user_id: the id
            :param kwargs:
            :return: Return UserOauthLogin
            """
            u = UserOauthLogin.get_by_provider(provider=provider,
                                               provider_user_id=provider_user_id)
            if u:
                return u
            return UserOauthLogin.create(user_id=self.id,
                                         provider=provider,
                                         provider_user_id=provider_user_id,
                                         **kwargs)

        def has_any_roles(self, *roles):
            """
            Check if user has any of the roles requested
            :param roles: tuple of roles string
            :return: bool
            """
            my_roles = [r.name for r in self.roles]
            roles = map(utils.slugify, list(roles))
            return any(i in my_roles for i in roles)

        def has_all_roles(self, *roles):
            """
            Check if user has ALL of the roles requested
            :param roles: tuple of roles string
            :return: bool
            """
            my_roles = [r.name for r in self.roles]
            roles = map(utils.slugify, roles)
            return all(i in my_roles for i in roles)

    class UserRoleMap(db.Model):
        user_id = db.Column(db.Integer, db.ForeignKey(User.id))
        role_id = db.Column(db.Integer, db.ForeignKey(UserRole.id))

        @classmethod
        def add(cls, user_id, role_id):
            r = cls.all().filter(cls.user_id == user_id)\
                .filter(cls.role_id == role_id)\
                .first()
            if not r:
                cls.create(user_id=user_id, role_id=role_id)

        @classmethod
        def remove(cls, user_id, role_id):
            r = cls.all().filter(cls.user_id == user_id)\
                .filter(cls.role_id == role_id)\
                .first()
            if r:
                r.delete(hard_delete=True)

    class UserOauthLogin(db.Model):
        user_id = db.Column(db.Integer, db.ForeignKey(User.id))
        provider = db.Column(db.String(50), index=True)
        provider_user_id = db.Column(db.String(250))
        name = db.Column(db.String(250))
        email = db.Column(db.String(250))
        profile_image_url = db.Column(db.String(250))
        access_token = db.Column(db.String(250))
        access_key_id = db.Column(db.String(250))
        access_secret_key = db.Column(db.String(250))
        link = db.Column(db.String(250))
        user = db.relationship(User, backref="oauth_logins")

        @classmethod
        def get_by_provider(cls, provider, provider_user_id):
            """
            Returns the entry of the provider and user id
            :params provider: str - the provider name
            :params provider_user_id: 
            """
            return cls.all()\
                .filter(cls.provider == provider)\
                .filter(cls.provider_user_id == provider_user_id)\
                .first()

    return utils.to_struct(User=User,
                           Role=UserRole,
                           RoleMap=UserRoleMap,
                           OauthLogin=UserOauthLogin)

# ------------------------------------------------------------------------------

def account(view, **kwargs):
    """
    This view is extendable

    kwargs:
        - on_signin_view
        - on_signout_view
        - template_dir

    """

    view_name = view.__name__
    model = kwargs.pop("model")
    User = model.User.User

    nav_menu_context = dict(__module=view.__module__, __class=view.__name__)

    login_view = "UserAccount:login"
    on_signin_view = kwargs["on_signin_view"] if "on_signin_view" \
                                                 in kwargs else "Index:index"
    on_signout_view = kwargs["on_signout_view"] if "on_signout_view" \
                                                   in kwargs else "Index:index"
    template_dir = kwargs["template_dir"] if "template_dir" \
                                             in kwargs else "WebPortfolio/Module/User/Account"
    template_page = template_dir + "/%s.html"

    login_manager = LoginManager()
    login_manager.login_view = login_view
    login_manager.login_message_category = "error"
    WebPortfolio.bind(login_manager.init_app)

    @login_manager.user_loader
    def load_user(userid):
        return User.get(userid)

    class Account(object):
        decorators = view.decorators + [login_required]

        SESSION_KEY_SET_EMAIL_DATA = "set_email_tmp_data"
        TEMP_DATA_KEY = "login_tmp_data"

        @property
        def tmp_data(self):
            return session[self.TEMP_DATA_KEY]

        @tmp_data.setter
        def tmp_data(self, data):
            session[self.TEMP_DATA_KEY] = data

        @classmethod
        def _login_enabled(cls):
            if not cls.config("MODULE_USER_ACCOUNT_ENABLE_LOGIN"):
                abort(403)

        @classmethod
        def _signup_enabled(cls):
            if not cls.config("MODULE_USER_ACCOUNT_ENABLE_SIGNUP"):
                abort(403)

        @classmethod
        def login_user(cls, user):
            login_user(user)
            now = datetime.datetime.now()
            user.update(last_login=now, last_visited=now)

        @classmethod
        def _oauth_enabled(cls):
            if not cls.config("MODULE_USER_ACCOUNT_ENABLE_OAUTH_LOGIN"):
                abort(403)

        @nav_menu("Login",
                  endpoint="UserAccount:login",
                  show=user_not_authenticated, **nav_menu_context)
        @route("login/", methods=["GET", "POST"], endpoint="UserAccount:login")
        @no_login_required
        def login(self):
            """ Login page """

            self._login_enabled()
            logout_user()
            self.tmp_data = None
            self.page_meta(title="Login")

            if request.method == "POST":
                email = request.form.get("email").strip()
                password = request.form.get("password").strip()

                if not email or not password:
                    flash_error("Email or Password is empty")
                    return redirect(url_for(login_view, next=request.form.get("next")))

                user = User.get_by_email(email)
                if user and user.password_matched(password):
                    self.login_user(user)
                    return redirect(request.form.get("next") or url_for(on_signin_view))
                else:
                    flash_error("Email or Password is invalid")
                    return redirect(url_for(login_view, next=request.form.get("next")))

            return self.render(login_url_next=request.args.get("next", ""),
                               login_url_default=url_for(on_signin_view),
                               signup_enabled=self.config("MODULE_USER_ACCOUNT_ENABLE_SIGNUP"),
                               oauth_enabled=self.config("MODULE_USER_ACCOUNT_ENABLE_OAUTH_LOGIN"),
                               view_template=template_page % "login")

        @nav_menu("Logout",
                  endpoint="UserAccount:logout",
                  show=user_authenticated,
                  order=100, **nav_menu_context)
        @route("logout/", endpoint="UserAccount:logout")
        @no_login_required
        def logout(self):
            logout_user()
            return redirect(url_for(on_signout_view or login_view))

        @nav_menu("Signup",
                  endpoint="UserAccount:signup",
                  show=[user_not_authenticated], **nav_menu_context)
        @route("signup/", methods=["GET", "POST"], endpoint="UserAccount:signup")
        @no_login_required
        def signup(self):
            """
            For Email Signup
            :return:
            """
            self._login_enabled()
            self._signup_enabled()
            self.page_meta(title="Signup")

            if request.method == "POST":
                # reCaptcha
                if not recaptcha.verify():
                    flash_error("Invalid Security code")
                    return redirect(url_for("UserAccount:signup",
                                            next=request.form.get("next")))
                try:
                    name = request.form.get("name")
                    email = request.form.get("email")
                    password = request.form.get("password")
                    password2 = request.form.get("password2")
                    profile_image_url = request.form.get("profile_image_url", None)

                    if not name:
                        raise ViewError("Name is required")
                    elif not utils.is_valid_email(email):
                        raise ViewError("Invalid email address '%s'" % email)
                    elif not password.strip() or password.strip() != password2.strip():
                        raise ViewError("Passwords don't match")
                    elif not utils.is_valid_password(password):
                        raise ViewError("Invalid password")
                    else:
                        new_account = User.new(email=email,
                                        password=password.strip(),
                                        first_name=name,
                                        profile_image_url=profile_image_url,
                                        signup_method="email")

                        self.login_user(new_account)
                        return redirect(request.form.get("next") or url_for(on_signin_view))
                except Exception as ex:
                    flash_error(ex.message)
                return redirect(url_for("UserAccount:signup",
                                        next=request.form.get("next")))

            logout_user()
            return self.render(login_url_next=request.args.get("next", ""),
                               view_template=template_page % "signup")

        @route("lost-password/",
               methods=["GET", "POST"],
               endpoint="UserAccount:lost_password")
        @no_login_required
        def lost_password(self):
            self._login_enabled()
            logout_user()

            self.page_meta(title="Lost Password")

            if request.method == "POST":
                email = request.form.get("email")
                user = User.get_by_email(email)
                if user:
                    delivery = self.config("MODULE_USER_ACCOUNT_RESET_PASSWORD_METHOD")

                    new_password = None
                    if delivery.upper() == "TOKEN":
                        token = user.set_temp_login()
                        url = url_for("UserAccount:reset_password",
                                      token=token,
                                      _external=True)
                    else:
                        new_password = user.set_password(password=None, random=True)
                        url = url_for("UserAccount:login", _external=True)

                    mailer.send_template("reset-password.txt",
                                         method_=delivery,
                                         to=user.email,
                                         name=user.email,
                                         url=url,
                                         new_password=new_password)

                    flash_success("A new password has been sent to '%s'" % email)
                else:
                    flash_error("Invalid email address")
                return redirect(url_for(login_view))
            else:
                return self.render(view_template=template_page % "lost_password")


        @nav_menu("Account Settings",
                  endpoint="UserAccount:account_settings",
                  order=99,
                  show=user_authenticated, **nav_menu_context)
        @route("account-settings",
               methods=["GET", "POST"],
               endpoint="UserAccount:account_settings")
        @fresh_login_required
        def account_settings(self):
            self.page_meta(title="Account Settings")

            if request.method == "POST":
                action = request.form.get("action")
                try:
                    action = action.lower()
                    #
                    if action == "info":
                        first_name = request.form.get("first_name").strip()
                        last_name = request.form.get("last_name", "").strip()

                        data = {
                            "first_name": first_name,
                            "last_name": last_name
                        }
                        current_user.update(**data)
                        flash_success("Account info updated successfully!")
                    #
                    elif action == "login":
                        confirm_password = request.form.get("confirm-password").strip()
                        if current_user.password_matched(confirm_password):
                            self.change_login_handler()
                            flash_success("Login Info updated successfully!")
                        else:
                            flash_error("Invalid password")
                    #
                    elif action == "password":
                        confirm_password = request.form.get("confirm-password").strip()
                        if current_user.password_matched(confirm_password):
                            self.change_password_handler()
                            flash_success("Password updated successfully!")
                        else:
                            flash_error("Invalid password")

                    elif action == "profile-photo":
                        file = request.files.get("file")
                        if file:
                            prefix = "profile-photos/%s/" % current_user.id
                            extensions = ["jpg", "jpeg", "png", "gif"]
                            my_photo = storage.upload(file,
                                                      prefix=prefix,
                                                      allowed_extensions=extensions)
                            if my_photo:
                                url = my_photo.url
                                current_user.update(profile_image_url=url)
                                flash_success("Profile Image updated successfully!")
                    else:
                        raise ViewError("Invalid action")

                except Exception as e:
                    flash_error(e.message)

                return redirect(url_for("UserAccount:account_settings"))

            return self.render(view_template=template_page % "account_settings")

        @classmethod
        def change_login_handler(cls, user_context=None, email=None):
            if not user_context:
                user_context = current_user
            if not email:
                email = request.form.get("email").strip()

            if not utils.is_valid_email(email):
                raise UserWarning("Invalid email address '%s'" % email)
            else:
                if email != user_context.email and User.get_by_email(email):
                    raise UserWarning("Email exists already '%s'" % email)
                elif email != user_context.email:
                    user_context.update(email=email)
                    return True
            return False

        @classmethod
        def change_password_handler(cls, user_context=None, password=None,
                                    password2=None):
            if not user_context:
                user_context = current_user
            if not password:
                password = request.form.get("password").strip()
            if not password2:
                password2 = request.form.get("password2").strip()

            if password:
                if password != password2:
                    raise UserWarning("Password don't match")
                elif not utils.is_valid_password(password):
                    raise UserWarning("Invalid password")
                else:
                    user_context.set_password(password)
                    return True
            else:
                raise UserWarning("Password is empty")


        # OAUTH Login
        @route("oauth-login/<provider>", methods=["GET", "POST"], endpoint="UserAccount:oauth_login")
        @no_login_required
        def oauth_login(self, provider):
            """ Login via oauth providers """

            self._login_enabled()
            self._oauth_enabled()

            provider = provider.lower()
            result = oauth.login(provider)
            response = oauth.response
            popup_js_custom = {
                "action": "",
                "url": ""
            }

            if result:
                if result.error:
                    pass

                elif result.user:
                    result.user.update()

                    oauth_user = result.user
                    user = User.get_by_oauth(provider=provider,
                                             provider_user_id=oauth_user.id)
                    if not user:
                        if oauth_user.email and User.get_by_email(oauth_user.email):
                            flash_error("Account already exists with this email '%s'. "
                                        "Try to login or retrieve your password " % oauth_user.email)

                            popup_js_custom.update({
                                "action": "redirect",
                                "url": url_for(login_view, next=request.form.get("next"))
                            })

                        else:
                            tmp_data = {
                                "is_oauth": True,
                                "provider": provider,
                                "id": oauth_user.id,
                                "name": oauth_user.name,
                                "picture": oauth_user.picture,
                                "first_name": oauth_user.first_name,
                                "last_name": oauth_user.last_name,
                                "email": oauth_user.email,
                                "link": oauth_user.link
                            }
                            if not oauth_user.email:
                                self.tmp_data = tmp_data

                                popup_js_custom.update({
                                    "action": "redirect",
                                    "url": url_for("UserAccount:setup_login")
                                })

                            else:
                                try:
                                    picture = oauth_user.picture
                                    user = User.new(email=oauth_user.email,
                                                    name=oauth_user.name,
                                                    signup_method=provider,
                                                    profile_image_url=picture
                                                    )
                                    user.add_oauth(provider,
                                                   oauth_user.provider_id,
                                                   name=oauth_user.name,
                                                   email=oauth_user.email,
                                                   profile_image_url=oauth_user.picture,
                                                   link=oauth_user.link)
                                except ModelError as e:
                                    flash_error(e.message)
                                    popup_js_custom.update({
                                        "action": "redirect",
                                        "url": url_for("UserAccount:login")
                                    })
                    if user:
                        self.login_user(user)

                    return self.render(popup_js=result.popup_js(custom=popup_js_custom),
                                       view_template=template_page % "oauth_login")
            return response

        @route("setup-login/", methods=["GET", "POST"], endpoint="UserAccount:setup_login")
        def setup_login(self):
            """
            Allows to setup a email password if it's not provided specially
            coming from oauth-login
            :return:
            """
            self._login_enabled()
            self.page_meta(title="Setup  Login")

            # Only user without email can set email
            if current_user.is_authenticated() and current_user.email:
                return redirect(url_for("%s:account_settings" % view_name))

            if self.tmp_data:
                if request.method == "POST":
                    if not self.tmp_data["is_oauth"]:
                        return redirect("UserAccount:login")

                    try:
                        email = request.form.get("email")
                        password = request.form.get("password")
                        password2 = request.form.get("password2")

                        if not utils.is_valid_email(email):
                            raise ViewError("Invalid email address '%s'" % email)
                        elif User.get_by_email(email):
                            raise ViewError("An account exists already with this email address '%s' " % email)
                        elif not password.strip() or password.strip() != password2.strip():
                            raise ViewError("Passwords don't match")
                        elif not utils.is_valid_password(password):
                            raise ViewError("Invalid password")
                        else:
                            user = User.new(email=email,
                                            password=password.strip(),
                                            name=self.tmp_data["name"],
                                            profile_image_url=self.tmp_data["picture"],
                                            signup_method=self.tmp_data["provider"])

                            user.add_oauth(self.tmp_data["provider"],
                                           self.tmp_data["id"],
                                           name=self.tmp_data["name"],
                                           email=email,
                                           profile_image_url=self.tmp_data["picture"],
                                           link=self.tmp_data["link"])

                            self.login_user(user)
                            self.tmp_data = None

                        return redirect(request.form.get("next") or url_for(on_signin_view))
                    except Exception as ex:
                        flash_error(ex.message)
                        return redirect(url_for("UserAccount:setup_login"))

                return self.render(provider=self.tmp_data,
                                   view_template=template_page % "setup_login")

            else:
                return redirect(url_for("UserAccount:login"))

        @route("reset-password/<token>",
               methods=["GET", "POST"],
               endpoint="UserAccount:reset_password")
        @no_login_required
        def reset_password(self, token):
            self._login_enabled()
            logout_user()

            self.page_meta(title="Reset Password")
            user = User.get_by_temp_login(token)
            if user:
                if not user.has_temp_login:
                    return redirect(url_for(on_signin_view))
                if request.method == "POST":
                    try:
                        self.change_password_handler(user_context=user)
                        user.clear_temp_login()
                        flash_success("Password updated successfully!")
                        return redirect(url_for(on_signin_view))
                    except Exception as ex:
                        flash_error("Error: %s" % ex.message)
                        return redirect(url_for("UserAccount:reset_password", token=token))
                else:
                    return self.render(token=token,
                                       view_template=template_page % "reset_password")
            else:
                abort(404, "Invalid token")

        @route("oauth-connect", methods=["POST"], endpoint="UserAccount:oauth_connect")
        def oauth_connect(self):
            """ To login via social """
            email = request.form.get("email").strip()
            name = request.form.get("name").strip()
            provider = request.form.get("provider").strip()
            provider_user_id = request.form.get("provider_user_id").strip()
            image_url = request.form.get("image_url").strip()
            next = request.form.get("next", "")
            try:
                current_user.oauth_connect(provider=provider,
                                         provider_user_id=provider_user_id,
                                         email=email,
                                         name=name,
                                         image_url=image_url)
            except Exception as ex:
                flash_error("Unable to link your account")

            return redirect(url_for("%s:account_settings" % view_name))

    return Account


# ------------------------------------------------------------------------------
# ADMIN

def admin(view, **kwargs):
    
    route_base = "user-admin"
    menu_name = "User Admin"

    model = kwargs.get("model")
    User = model.User.User
    Role = model.User.Role
    RoleMap = model.User.RoleMap

    template_dir = kwargs.get("template_dir", "WebPortfolio/Module/User/Admin")
    template_page = template_dir + "/%s.html"

    # Create a Admin menu for all the methods in Admin
    @nav_menu(menu_name, group="admin")
    class NavMenu(object): pass
    # The nav_menu_context helps attach all the methods to NavMenu
    nav_menu_context = dict(__module=NavMenu.__module__,
                            __class=NavMenu.__name__)
    
    class Admin(object):
        decorators = view.decorators + [login_required]

        @classmethod
        def _validate_admin_roles(cls, user):
            admin = current_user

        @classmethod
        def _user_roles_options(cls):
            return [(r.id, r.name) for r in Role.all().order_by(Role.level.desc())]

        @nav_menu("All Users", endpoint="UserAdmin:index", order=1, **nav_menu_context)
        @route("%s/" % route_base, endpoint="UserAdmin:index")
        def user_admin_index(self):

            self.page_meta(title="Users - User Admin")
            per_page = self.config("APPLICATION_PAGINATION_PER_PAGE", 25)

            page = request.args.get("page", 1)
            include_deleted = True if request.args.get("include-deleted") == "y" else False
            name = request.args.get("name")
            email = request.args.get("email")
            role = request.args.get("role")
            sorting = request.args.get("sorting", "first_name__asc")

            users = User.all(include_deleted=include_deleted)

            if name:
                users = users.filter(User.name.contains(name))
            if email:
                users = users.filter(User.email.contains(email))
            if role:
                users = users.join(RoleMap)\
                    .join(Role)\
                    .filter(Role.id == role)
            if sorting and "__" in sorting:
                col, dir = sorting.split("__", 2)
                users = users.order_by(getattr(User, col) + " %s" % dir)

            users = users.paginate(page=page, per_page=per_page)

            sorting = [("first_name__asc", "Name ASC"),
                       ("first_name__desc", "Name DESC"),
                       ("email__asc", "Email ASC"),
                       ("email__desc", "Email DESC"),
                       ("created_at__asc", "Signup ASC"),
                       ("created_at__desc", "Signup Desc"),
                       ("last_login__asc", "Login ASC"),
                       ("last_login__desc", "Login Desc")]
            return self.render(user_roles_options=self._user_roles_options(),
                               sorting_options=sorting,
                               users=users,
                               search_query={
                                   "include-deleted": request.args.get("include-deleted", "n"),
                                   "role": int(request.args.get("role")) if request.args.get("role") else "",
                                   "status": request.args.get("status"),
                                   "first_name": request.args.get("name", ""),
                                   "email": request.args.get("email", ""),
                                   "sorting": request.args.get("sorting")},
                               view_template=template_page % "index")

        @nav_menu("User Roles", endpoint="UserAdmin:roles", order=2, **nav_menu_context)
        @route("%s/roles" % route_base, methods=["GET", "POST"], endpoint="UserAdmin:roles")
        def user_admin_roles(self):
            """
            Only admin and super admin can add/remove roles
            RESTRICTED ROLES CAN'T BE CHANGED
            """
            roles_rage_max = 11
            if request.method == "POST":
                try:
                    id = request.form.get("id")
                    name = request.form.get("name")
                    level = request.form.get("level")
                    action = request.form.get("action")

                    if name and level:
                        level = int(level)
                        name = name.upper()
                        _levels = [r[0] for r in Role.PRIMARY]
                        _names = [r[1] for r in Role.PRIMARY]
                        if level in _levels or name in _names:
                            raise ViewError("Can't modify PRIMARY Roles - name: %s, level: %s " % (name, level))
                        else:
                            if id:
                                role = Role.get(id)
                                if role:
                                    if action == "delete":
                                        role.delete()
                                        flash_success("Role '%s' deleted successfully!" % role.name)
                                    elif action == "update":
                                        if role.level != level and Role.get_by_level(level):
                                            raise ViewError("Role Level '%s' exists already" % level)
                                        elif role.name != name and Role.get_by_name(name):
                                            raise ViewError("Role Name '%s'  exists already" % name)
                                        else:
                                            role.update(name=name, level=level)
                                            flash_success("Role '%s (%s)' updated successfully" % (name, level))
                                else:
                                    raise ViewError("Role doesn't exist")
                            else:
                                if Role.get_by_level(level):
                                    raise ViewError("Role Level '%s' exists already" % level)
                                elif Role.get_by_name(name):
                                    raise ViewError("Role Name '%s'  exists already" % name)
                                else:
                                    Role.new(name=name, level=level)
                                    flash_success("New Role '%s (%s)' addedd successfully" % (name, level))
                except Exception as ex:
                    flash_error("Error: %s" % ex.message)
                return redirect(url_for("UserAdmin:roles"))
            else:
                self.page_meta(title="User Roles - Users Admin")
                roles = Role.all().order_by(Role.level.desc())

                allocated_levels = [r.level for r in roles]
                levels_options = [(l, l) for l in range(1, roles_rage_max) if l not in allocated_levels]

                return self.render(roles=roles,
                                   levels_options=levels_options,
                                   view_template=template_page % "roles")

        @nav_menu("Info", endpoint="UserAdmin:get", show=False, **nav_menu_context)
        @route("%s/<id>" % route_base, endpoint="UserAdmin:get")
        def user_admin_get(self, id):
            self.page_meta(title="User Info - Users Admin")
            user = User.get(id, include_deleted=True)
            if not user:
                abort(404, "User doesn't exist")

            user_roles = [r.id for r in user.roles]
            user_roles_name = [r.name for r in user.roles]

            return self.render(user=user,
                               user_roles_name=user_roles_name,
                               user_roles=user_roles,
                               user_roles_options=self._user_roles_options(),
                               view_template=template_page % "get")

        @route("%s/post" % route_base, methods=["POST"], endpoint="UserAdmin:post")
        def user_admin_post(self):
            try:
                id = request.form.get("id")
                user = User.get(id, include_deleted=True)
                if not user:
                    flash_error("Can't change user info. Invalid user")
                    return redirect(url_for("UserAdmin:index"))

                email = request.form.get("email", "").strip()
                first_name = request.form.get("first_name")
                last_name = request.form.get("last_name")
                user_roles = request.form.getlist("user_roles")
                action = request.form.get("action")

                if action == "activate":
                    user.update(active=True)
                    flash_success("User has been ACTIVATED")
                elif action == "deactivate":
                    user.update(active=False)
                    flash_success("User is now DEACTIVATED")
                elif action == "delete":
                    user.delete()
                    flash_success("User has been deleted")
                elif action == "undelete":
                    user.delete(False)
                    flash_success("User is now active")
                else:
                    if email and email != user.email:
                        if not utils.is_valid_email(email):
                            raise ViewError("Invalid email address '%s'" % email)
                        else:
                            if User.get_by_email(email):
                                raise ViewError("Email exists already '%s'" % email)
                            user.update(email=email)

                    user.update(first_name=first_name, last_name=last_name)

                    if user_roles:
                        user_roles = map(int, user_roles)
                        user.update_roles(user_roles)

                    flash_success("User's Info updated successfully!")
            except Exception as ex:
                flash_error("Error: %s " % ex.message)
            return redirect(url_for("UserAdmin:get", id=id))

        @route("%s/reset-password" % route_base, methods=["POST"], endpoint="UserAdmin:reset_password")
        def user_admin_reset_password(self):
            """
            Reset the password
            :returns string: The new password string
            """
            try:
                id = request.form.get("id")
                user = User.get(id)
                if not user:
                    raise ViewError("Invalid User")

                method_ = self.config("LOGIN_RESET_PASSWORD_METHOD", "").upper()
                new_password = None
                if method_ == "TOKEN":
                    token = user.set_temp_login()
                    url = url_for("UserAccount:temp_login_token",
                                  token=token,
                                  _external=True)
                else:
                    new_password = user.set_password(password=None, random=True)
                    url = url_for("UserAccount:login", _external=True)

                mailer.send_template("reset-password.txt",
                                     method_=method_,
                                     to=user.email,
                                     name=user.email,
                                     url=url,
                                     new_password=new_password)

                flash_success("Password Reset instruction is sent to email")
            except Exception as ex:
                flash_error("Error: %s " % ex.message)
            return redirect(url_for("UserAdmin:get", id=id))

        @route("%s/create" % route_base, methods=["POST"], endpoint="UserAdmin:create")
        def user_admin_create(self):
            try:
                email = request.form.get("email")
                first_name = request.form.get("first_name")
                last_name = request.form.get("last_name")
                user_roles = request.form.getlist("user_roles")

                if not first_name:
                    raise ViewError("First Name is required")
                elif not email:
                    raise ViewError("Email is required")
                elif not utils.is_valid_email(email):
                    raise ViewError("Invalid email address")
                if User.get_by_email(email):
                    raise ViewError("Email '%s' exists already" % email)
                else:
                    user = User.new(email=email,
                                    first_name=first_name,
                                    last_name=last_name,
                                    signup_method="email-from-admin")
                    if user:
                        flash_success("User created successfully!")

                        if user_roles:
                            user.update_roles(map(int, user_roles))

                        return redirect(url_for("UserAdmin:get", id=user.id))
                    else:
                        raise ViewError("Couldn't create new user")
            except Exception as ex:
                flash_error("Error: %s" % ex.message)
            return redirect(url_for("UserAdmin:index"))

    return Admin

